@@ -59,15 +59,17 @@ public interface ConditionalWriter extends AutoCloseable {
 
     public Status getStatus() throws AccumuloException, AccumuloSecurityException {
       if (status == null) {
-        if (exception instanceof AccumuloException)
+        if (exception instanceof AccumuloException) {
           throw new AccumuloException(exception);
+        }
         if (exception instanceof AccumuloSecurityException) {
           AccumuloSecurityException ase = (AccumuloSecurityException) exception;
           throw new AccumuloSecurityException(ase.getUser(),
               SecurityErrorCode.valueOf(ase.getSecurityErrorCode().name()), ase.getTableInfo(),
               ase);
-        } else
+        } else {
           throw new AccumuloException(exception);
+        }
       }
 
       return status;
@@ -105,7 +107,7 @@ public interface ConditionalWriter extends AutoCloseable {
      */
     VIOLATED,
     /**
-     * error occurred after mutation was sent to server, its unknown if the mutation was written.
+     * Error occurred after mutation was sent to server, its unknown if the mutation was written.
      * Although the status of the mutation is unknown, Accumulo guarantees the mutation will not be
      * written at a later point in time.
      */
