@@ -102,7 +102,7 @@ public class ServiceLock implements Watcher {
   private LockWatcher lockWatcher;
   private String lockNodeName;
   private volatile boolean lockWasAcquired;
-  private volatile boolean watchingParent = false;
+  private volatile boolean watchingParent;
 
   private String createdNodeName;
   private String watchingNodeName;
@@ -190,14 +190,18 @@ public class ServiceLock implements Watcher {
     children.forEach(c -> {
       LOG.trace("Validating {}", c);
       if (c.startsWith(ZLOCK_PREFIX)) {
-        String candidate = c.substring(ZLOCK_PREFIX.length() + 1);
+        String candidate = c.substring(ZLOCK_PREFIX.length());
         if (candidate.contains("#")) {
           int idx = candidate.indexOf('#');
-          String uuid = candidate.substring(0, idx - 1);
+          String uuid = candidate.substring(0, idx);
           String sequenceNum = candidate.substring(idx + 1);
           try {
             LOG.trace("Testing uuid format of {}", uuid);
-            UUID.fromString(uuid);
+            // string check guards uuids like "1-1-1-1-1" that parse to
+            // "00000001-0001-0001-0001-000000000001"
+            if (!uuid.equals(UUID.fromString(uuid).toString())) {
+              throw new IllegalArgumentException(uuid + " is an invalid UUID");
+            }
             if (sequenceNum.length() == 10) {
               try {
                 LOG.trace("Testing number format of {}", sequenceNum);
@@ -277,7 +281,7 @@ public class ServiceLock implements Watcher {
 
     List<String> children = validateAndSort(path, zooKeeper.getChildren(path.toString(), null));
 
-    if (null == children || !children.contains(createdEphemeralNode)) {
+    if (!children.contains(createdEphemeralNode)) {
       LOG.error("Expected ephemeral node {} to be in the list of children {}", createdEphemeralNode,
           children);
       throw new RuntimeException(
@@ -405,8 +409,7 @@ public class ServiceLock implements Watcher {
       // were created but the client missed the response for some reason. Find the ephemeral nodes
       // with this ZLOCK_UUID and lowest sequential number.
       List<String> children = validateAndSort(path, zooKeeper.getChildren(path.toString(), null));
-      if (null == children
-          || !children.contains(createPath.substring(path.toString().length() + 1))) {
+      if (!children.contains(createPath.substring(path.toString().length() + 1))) {
         LOG.error("Expected ephemeral node {} to be in the list of children {}", createPath,
             children);
         throw new RuntimeException("Lock attempt ephemeral node no longer exist " + createPath);
@@ -443,6 +446,9 @@ public class ServiceLock implements Watcher {
           }
         }
       }
+      if (lowestSequentialPath == null) {
+        throw new IllegalStateException("Could not find lowest sequential path under " + path);
+      }
       final String pathForWatcher = lowestSequentialPath;
 
       // Set a watcher on the lowest sequential node that we created, this handles the case
@@ -633,7 +639,7 @@ public class ServiceLock implements Watcher {
     var zLockPath = path(lid.path);
     List<String> children = validateAndSort(zLockPath, zc.getChildren(zLockPath.toString()));
 
-    if (children == null || children.isEmpty()) {
+    if (children.isEmpty()) {
       return false;
     }
 
@@ -651,7 +657,7 @@ public class ServiceLock implements Watcher {
 
     List<String> children = validateAndSort(path, zk.getChildren(path.toString(), null));
 
-    if (children == null || children.isEmpty()) {
+    if (children.isEmpty()) {
       return null;
     }
 
@@ -665,7 +671,7 @@ public class ServiceLock implements Watcher {
 
     List<String> children = validateAndSort(path, zc.getChildren(path.toString()));
 
-    if (children == null || children.isEmpty()) {
+    if (children.isEmpty()) {
       return null;
     }
 
@@ -682,7 +688,7 @@ public class ServiceLock implements Watcher {
 
     List<String> children = validateAndSort(path, zc.getChildren(path.toString()));
 
-    if (children == null || children.isEmpty()) {
+    if (children.isEmpty()) {
       return 0;
     }
 
@@ -714,7 +720,7 @@ public class ServiceLock implements Watcher {
 
     List<String> children = validateAndSort(path, zk.getChildren(path.toString()));
 
-    if (children == null || children.isEmpty()) {
+    if (children.isEmpty()) {
       throw new IllegalStateException("No lock is held at " + path);
     }
 
@@ -735,7 +741,7 @@ public class ServiceLock implements Watcher {
 
     List<String> children = validateAndSort(path, zk.getChildren(path.toString()));
 
-    if (children == null || children.isEmpty()) {
+    if (children.isEmpty()) {
       throw new IllegalStateException("No lock is held at " + path);
     }
 
