@@ -373,7 +373,9 @@ public class TabletMetadata {
         future = location;
       }
       // only care about the state so don't need walogs and chopped params
-      var tls = new TabletLocationState(extent, future, current, last, suspend, null, false);
+      // Use getExtent() when passing the extent as the private reference may not have been
+      // initialized yet. This will also ensure PREV_ROW was fetched
+      var tls = new TabletLocationState(getExtent(), future, current, last, suspend, null, false);
       return tls.getState(liveTServers);
     } catch (TabletLocationState.BadLocationStateException blse) {
       throw new IllegalArgumentException("Error creating TabletLocationState", blse);
