@@ -319,6 +319,12 @@ public class TabletsMetadata implements Iterable<TabletMetadata>, AutoCloseable
       return this;
     }
 
+    /**
+     * For a given data level, read all of its tablets metadata. For {@link DataLevel#USER} this
+     * will read tablet metadata from the accumulo.metadata table for all user tables. For
+     * {@link DataLevel#METADATA} this will read tablet metadata from the accumulo.root table. For
+     * {@link DataLevel#ROOT} this will read tablet metadata from Zookeeper.
+     */
     @Override
     public Options forLevel(DataLevel level) {
       this.level = level;
@@ -326,6 +332,13 @@ public class TabletsMetadata implements Iterable<TabletMetadata>, AutoCloseable
       return this;
     }
 
+    /**
+     * For a given table read all of its tablet metadata. If the table id is for a user table, then
+     * its metadata will be read from its section in the accumulo.metadata table. If the table id is
+     * for the accumulo.metadata table, then its metadata will be read from the accumulo.root table.
+     * If the table id is for the accumulo.root table, then its metadata will be read from
+     * zookeeper.
+     */
     @Override
     public TableRangeOptions forTable(TableId tableId) {
       this.level = DataLevel.of(tableId);
@@ -590,6 +603,6 @@ public class TabletsMetadata implements Iterable<TabletMetadata>, AutoCloseable
   }
 
   public Stream<TabletMetadata> stream() {
-    return StreamSupport.stream(tablets.spliterator(), false);
+    return StreamSupport.stream(tablets.spliterator(), false).onClose(this::close);
   }
 }
