@@ -31,6 +31,7 @@ import java.util.function.Predicate;
 
 import org.apache.accumulo.core.client.rfile.RFileSource;
 import org.apache.accumulo.core.client.summary.SummarizerConfiguration;
+import org.apache.accumulo.core.file.NoSuchMetaStoreException;
 import org.apache.accumulo.core.file.blockfile.impl.BasicCacheProvider;
 import org.apache.accumulo.core.file.blockfile.impl.CachableBlockFile;
 import org.apache.accumulo.core.file.blockfile.impl.CachableBlockFile.CachableBuilder;
@@ -225,10 +226,18 @@ public class SummaryReader {
   }
 
   public static void print(Reader iter, PrintStream out) throws IOException {
+    List<SummarySerializer> stores = null;
+    try {
+      stores = load(iter::getMetaStore, conf -> true);
+    } catch (NoSuchMetaStoreException e) {
+      out.print("No summary data present in file\n");
+      // exit method early since rfile contains no summary data
+      return;
+    }
+
     String indent = "   ";
     out.print("Summary data : \n");
 
-    List<SummarySerializer> stores = load(iter::getMetaStore, conf -> true);
     int i = 1;
     for (SummarySerializer summaryStore : stores) {
       out.printf("%sSummary %d of %d generated by : %s\n", indent, i, stores.size(),
