@@ -1,30 +1,31 @@
 /*
- * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
+ *   https://www.apache.org/licenses/LICENSE-2.0
  *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 package org.apache.accumulo.core.iterators.user;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertTrue;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 import java.security.SecureRandom;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.Map;
-import java.util.Random;
 import java.util.Set;
 import java.util.TreeMap;
 import java.util.concurrent.atomic.AtomicLong;
@@ -35,15 +36,16 @@ import org.apache.accumulo.core.data.Key;
 import org.apache.accumulo.core.data.Range;
 import org.apache.accumulo.core.data.Value;
 import org.apache.accumulo.core.iterators.SortedKeyValueIterator;
-import org.apache.accumulo.core.iterators.SortedMapIterator;
 import org.apache.accumulo.core.iterators.ValueFormatException;
+import org.apache.accumulo.core.iteratorsImpl.system.SortedMapIterator;
 import org.apache.hadoop.io.Text;
-import org.junit.AfterClass;
-import org.junit.BeforeClass;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterAll;
+import org.junit.jupiter.api.BeforeAll;
+import org.junit.jupiter.api.Test;
 
 public abstract class TestCfCqSlice {
 
+  private static final SecureRandom random = new SecureRandom();
   private static final Range INFINITY = new Range();
   private static final Lexicoder<Long> LONG_LEX = new ReadableLongLexicoder(4);
   private static final AtomicLong ROW_ID_GEN = new AtomicLong();
@@ -58,12 +60,12 @@ public abstract class TestCfCqSlice {
 
   private static TreeMap<Key,Value> data;
 
-  @BeforeClass
+  @BeforeAll
   public static void setupData() {
     data = createMap(LR_DIM, LR_DIM, LR_DIM);
   }
 
-  @AfterClass
+  @AfterAll
   public static void clearData() {
     data = null;
   }
@@ -75,8 +77,8 @@ public abstract class TestCfCqSlice {
     for (int i = 0; i < LR_DIM; i++) {
       for (int j = 0; j < LR_DIM; j++) {
         for (int k = 0; k < LR_DIM; k++) {
-          assertTrue("(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan",
-              foundKvs[i][j][k]);
+          assertTrue(foundKvs[i][j][k],
+              "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan");
         }
       }
     }
@@ -91,12 +93,11 @@ public abstract class TestCfCqSlice {
       for (int j = 0; j < LR_DIM; j++) {
         for (int k = 0; k < LR_DIM; k++) {
           if (rowId == i) {
-            assertTrue("(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan",
-                foundKvs[i][j][k]);
+            assertTrue(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan");
           } else {
-            assertFalse(
-                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan",
-                foundKvs[i][j][k]);
+            assertFalse(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan");
           }
         }
       }
@@ -110,10 +111,10 @@ public abstract class TestCfCqSlice {
     long sliceMinCq = 30;
     long sliceMaxCf = 25;
     long sliceMaxCq = 35;
-    assertTrue("slice param must be less than LR_DIM", sliceMinCf < LR_DIM);
-    assertTrue("slice param must be less than LR_DIM", sliceMinCq < LR_DIM);
-    assertTrue("slice param must be less than LR_DIM", sliceMaxCf < LR_DIM);
-    assertTrue("slice param must be less than LR_DIM", sliceMaxCq < LR_DIM);
+    assertTrue(sliceMinCf < LR_DIM, "slice param must be less than LR_DIM");
+    assertTrue(sliceMinCq < LR_DIM, "slice param must be less than LR_DIM");
+    assertTrue(sliceMaxCf < LR_DIM, "slice param must be less than LR_DIM");
+    assertTrue(sliceMaxCq < LR_DIM, "slice param must be less than LR_DIM");
     Map<String,String> opts = new HashMap<>();
     opts.put(CfCqSliceOpts.OPT_MIN_CF, new String(LONG_LEX.encode(sliceMinCf), UTF_8));
     opts.put(CfCqSliceOpts.OPT_MIN_CQ, new String(LONG_LEX.encode(sliceMinCq), UTF_8));
@@ -124,12 +125,11 @@ public abstract class TestCfCqSlice {
       for (int j = 0; j < LR_DIM; j++) {
         for (int k = 0; k < LR_DIM; k++) {
           if (j >= sliceMinCf && j <= sliceMaxCf && k >= sliceMinCq && k <= sliceMaxCq) {
-            assertTrue("(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan",
-                foundKvs[i][j][k]);
+            assertTrue(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan");
           } else {
-            assertFalse(
-                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan",
-                foundKvs[i][j][k]);
+            assertFalse(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan");
           }
         }
       }
@@ -153,12 +153,11 @@ public abstract class TestCfCqSlice {
       for (int j = 0; j < LR_DIM; j++) {
         for (int k = 0; k < LR_DIM; k++) {
           if (j == sliceMinCf && k == sliceMinCq) {
-            assertTrue("(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan",
-                foundKvs[i][j][k]);
+            assertTrue(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan");
           } else {
-            assertFalse(
-                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan",
-                foundKvs[i][j][k]);
+            assertFalse(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan");
           }
         }
       }
@@ -184,12 +183,11 @@ public abstract class TestCfCqSlice {
       for (int j = 0; j < LR_DIM; j++) {
         for (int k = 0; k < LR_DIM; k++) {
           if (j == 21 && k == 21) {
-            assertTrue("(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan",
-                foundKvs[i][j][k]);
+            assertTrue(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan");
           } else {
-            assertFalse(
-                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan",
-                foundKvs[i][j][k]);
+            assertFalse(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan");
           }
         }
       }
@@ -209,12 +207,11 @@ public abstract class TestCfCqSlice {
       for (int j = 0; j < LR_DIM; j++) {
         for (int k = 0; k < LR_DIM; k++) {
           if (k >= sliceMinCq && k <= sliceMaxCq) {
-            assertTrue("(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan",
-                foundKvs[i][j][k]);
+            assertTrue(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan");
           } else {
-            assertFalse(
-                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan",
-                foundKvs[i][j][k]);
+            assertFalse(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan");
           }
         }
       }
@@ -234,12 +231,11 @@ public abstract class TestCfCqSlice {
       for (int j = 0; j < LR_DIM; j++) {
         for (int k = 0; k < LR_DIM; k++) {
           if (j >= sliceMinCf && j <= sliceMaxCf) {
-            assertTrue("(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan",
-                foundKvs[i][j][k]);
+            assertTrue(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan");
           } else {
-            assertFalse(
-                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan",
-                foundKvs[i][j][k]);
+            assertFalse(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan");
           }
         }
       }
@@ -264,8 +260,8 @@ public abstract class TestCfCqSlice {
     for (int i = 0; i < LR_DIM; i++) {
       for (int j = 0; j < LR_DIM; j++) {
         for (int k = 0; k < LR_DIM; k++) {
-          assertFalse("(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan",
-              foundKvs[i][j][k]);
+          assertFalse(foundKvs[i][j][k],
+              "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan");
         }
       }
     }
@@ -280,27 +276,27 @@ public abstract class TestCfCqSlice {
     long sliceMaxCf = 25;
     long sliceMinCq = 30;
     long sliceMaxCq = 35;
-    assertTrue("slice param must be less than LR_DIM", sliceMinCf < LR_DIM);
-    assertTrue("slice param must be less than LR_DIM", sliceMinCq < LR_DIM);
-    assertTrue("slice param must be less than LR_DIM", sliceMaxCf < LR_DIM);
-    assertTrue("slice param must be less than LR_DIM", sliceMaxCq < LR_DIM);
+    assertTrue(sliceMinCf < LR_DIM, "slice param must be less than LR_DIM");
+    assertTrue(sliceMinCq < LR_DIM, "slice param must be less than LR_DIM");
+    assertTrue(sliceMaxCf < LR_DIM, "slice param must be less than LR_DIM");
+    assertTrue(sliceMaxCq < LR_DIM, "slice param must be less than LR_DIM");
     firstOpts.put(CfCqSliceOpts.OPT_MIN_CF, new String(LONG_LEX.encode(sliceMinCf), UTF_8));
     firstOpts.put(CfCqSliceOpts.OPT_MAX_CF, new String(LONG_LEX.encode(sliceMaxCf), UTF_8));
     secondOpts.put(CfCqSliceOpts.OPT_MIN_CQ, new String(LONG_LEX.encode(sliceMinCq), UTF_8));
     secondOpts.put(CfCqSliceOpts.OPT_MAX_CQ, new String(LONG_LEX.encode(sliceMaxCq), UTF_8));
-    SortedKeyValueIterator<Key,Value> skvi = getFilterClass().newInstance();
+    SortedKeyValueIterator<Key,Value> skvi =
+        getFilterClass().getDeclaredConstructor().newInstance();
     skvi.init(new SortedMapIterator(data), firstOpts, null);
     loadKvs(skvi.deepCopy(null), foundKvs, secondOpts, INFINITY);
     for (int i = 0; i < LR_DIM; i++) {
       for (int j = 0; j < LR_DIM; j++) {
         for (int k = 0; k < LR_DIM; k++) {
           if (j >= sliceMinCf && j <= sliceMaxCf && k >= sliceMinCq && k <= sliceMaxCq) {
-            assertTrue("(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan",
-                foundKvs[i][j][k]);
+            assertTrue(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan");
           } else {
-            assertFalse(
-                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan",
-                foundKvs[i][j][k]);
+            assertFalse(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan");
           }
         }
       }
@@ -314,10 +310,10 @@ public abstract class TestCfCqSlice {
     long sliceMinCq = 30;
     long sliceMaxCf = 25;
     long sliceMaxCq = 35;
-    assertTrue("slice param must be less than LR_DIM", sliceMinCf < LR_DIM);
-    assertTrue("slice param must be less than LR_DIM", sliceMinCq < LR_DIM);
-    assertTrue("slice param must be less than LR_DIM", sliceMaxCf < LR_DIM);
-    assertTrue("slice param must be less than LR_DIM", sliceMaxCq < LR_DIM);
+    assertTrue(sliceMinCf < LR_DIM, "slice param must be less than LR_DIM");
+    assertTrue(sliceMinCq < LR_DIM, "slice param must be less than LR_DIM");
+    assertTrue(sliceMaxCf < LR_DIM, "slice param must be less than LR_DIM");
+    assertTrue(sliceMaxCq < LR_DIM, "slice param must be less than LR_DIM");
     Map<String,String> opts = new HashMap<>();
     opts.put(CfCqSliceOpts.OPT_MIN_CF, new String(LONG_LEX.encode(sliceMinCf), UTF_8));
     opts.put(CfCqSliceOpts.OPT_MIN_INCLUSIVE, "false");
@@ -331,12 +327,11 @@ public abstract class TestCfCqSlice {
       for (int j = 0; j < LR_DIM; j++) {
         for (int k = 0; k < LR_DIM; k++) {
           if (j > sliceMinCf && j <= sliceMaxCf && k > sliceMinCq && k <= sliceMaxCq) {
-            assertTrue("(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan",
-                foundKvs[i][j][k]);
+            assertTrue(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan");
           } else {
-            assertFalse(
-                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan",
-                foundKvs[i][j][k]);
+            assertFalse(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan");
           }
         }
       }
@@ -354,12 +349,11 @@ public abstract class TestCfCqSlice {
       for (int j = 0; j < LR_DIM; j++) {
         for (int k = 0; k < LR_DIM; k++) {
           if (j > sliceMinCf && j <= sliceMaxCf && k > sliceMinCq && k <= sliceMaxCq) {
-            assertTrue("(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan",
-                foundKvs[i][j][k]);
+            assertTrue(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must be found in scan");
           } else {
-            assertFalse(
-                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan",
-                foundKvs[i][j][k]);
+            assertFalse(foundKvs[i][j][k],
+                "(r, cf, cq) == (" + i + ", " + j + ", " + k + ") must not be found in scan");
           }
         }
       }
@@ -373,19 +367,18 @@ public abstract class TestCfCqSlice {
   private void loadKvs(SortedKeyValueIterator<Key,Value> parent, boolean[][][] foundKvs,
       Map<String,String> options, Range range) {
     try {
-      SortedKeyValueIterator<Key,Value> skvi = getFilterClass().newInstance();
+      SortedKeyValueIterator<Key,Value> skvi =
+          getFilterClass().getDeclaredConstructor().newInstance();
       skvi.init(parent, options, null);
       skvi.seek(range, EMPTY_CF_SET, false);
 
-      Random random = new SecureRandom();
-
       while (skvi.hasTop()) {
         Key k = skvi.getTopKey();
         int row = LONG_LEX.decode(k.getRow().copyBytes()).intValue();
         int cf = LONG_LEX.decode(k.getColumnFamily().copyBytes()).intValue();
         int cq = LONG_LEX.decode(k.getColumnQualifier().copyBytes()).intValue();
 
-        assertFalse("Duplicate " + row + " " + cf + " " + cq, foundKvs[row][cf][cq]);
+        assertFalse(foundKvs[row][cf][cq], "Duplicate " + row + " " + cf + " " + cq);
         foundKvs[row][cf][cq] = true;
 
         if (random.nextInt(100) == 0) {
