@@ -1,26 +1,28 @@
 /*
- * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
+ *   https://www.apache.org/licenses/LICENSE-2.0
  *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 package org.apache.accumulo.core.iterators.user;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertTrue;
-import static org.junit.Assert.fail;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 import java.io.IOException;
 import java.util.ArrayList;
@@ -44,29 +46,27 @@ import org.apache.accumulo.core.data.Value;
 import org.apache.accumulo.core.iterators.IteratorEnvironment;
 import org.apache.accumulo.core.iterators.IteratorUtil.IteratorScope;
 import org.apache.accumulo.core.iterators.SortedKeyValueIterator;
-import org.apache.accumulo.core.iterators.SortedMapIterator;
 import org.apache.accumulo.core.iterators.WrappingIterator;
-import org.apache.accumulo.core.iterators.system.ColumnFamilySkippingIterator;
-import org.apache.accumulo.core.iterators.system.VisibilityFilter;
+import org.apache.accumulo.core.iteratorsImpl.system.ColumnFamilySkippingIterator;
+import org.apache.accumulo.core.iteratorsImpl.system.SortedMapIterator;
+import org.apache.accumulo.core.iteratorsImpl.system.VisibilityFilter;
 import org.apache.accumulo.core.security.Authorizations;
 import org.apache.accumulo.core.security.ColumnVisibility;
 import org.apache.hadoop.io.Text;
 import org.easymock.EasyMock;
-import org.junit.Before;
-import org.junit.Test;
-
-import com.google.common.collect.ImmutableMap;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 
 public class TransformingIteratorTest {
 
   private static Authorizations authorizations =
       new Authorizations("vis0", "vis1", "vis2", "vis3", "vis4");
-  private static final Map<String,String> EMPTY_OPTS = ImmutableMap.of();
+  private static final Map<String,String> EMPTY_OPTS = Map.of();
   private TransformingIterator titer;
 
   private TreeMap<Key,Value> data = new TreeMap<>();
 
-  @Before
+  @BeforeEach
   public void createData() {
     data.clear();
     generateRow(data, "row1");
@@ -88,8 +88,8 @@ public class TransformingIteratorTest {
     ReuseIterator reuserIter = new ReuseIterator();
     reuserIter.init(visFilter, EMPTY_OPTS, null);
     try {
-      titer = clazz.newInstance();
-    } catch (InstantiationException | IllegalAccessException e) {
+      titer = clazz.getDeclaredConstructor().newInstance();
+    } catch (ReflectiveOperationException e) {
       throw new RuntimeException(e);
     }
 
@@ -104,7 +104,7 @@ public class TransformingIteratorTest {
           new Authorizations("vis0", "vis1", "vis2", "vis3"));
       opts = cfg.getOptions();
     } else {
-      opts = ImmutableMap.of();
+      opts = Map.of();
     }
     titer.init(reuserIter, opts, iterEnv);
   }
@@ -143,7 +143,7 @@ public class TransformingIteratorTest {
       setUpTransformIterator(clazz);
 
       // All rows with visibilities reversed
-      TransformingIterator iter = clazz.newInstance();
+      TransformingIterator iter = clazz.getDeclaredConstructor().newInstance();
       TreeMap<Key,Value> expected = new TreeMap<>();
       for (int row = 1; row <= 3; ++row) {
         for (int cf = 1; cf <= 3; ++cf) {
@@ -229,9 +229,11 @@ public class TransformingIteratorTest {
     // be inside the range.
 
     TreeMap<Key,Value> expected = new TreeMap<>();
-    for (int cq = 1; cq <= 3; ++cq)
-      for (int cv = 1; cv <= 3; ++cv)
+    for (int cq = 1; cq <= 3; ++cq) {
+      for (int cv = 1; cv <= 3; ++cv) {
         putExpected(expected, 1, 3, cq, cv, PartialKey.ROW);
+      }
+    }
     checkExpected(new Range(new Key("row1", "cf0"), true, new Key("row1", "cf1"), false), expected);
   }
 
@@ -276,10 +278,13 @@ public class TransformingIteratorTest {
     setUpTransformIterator(ColFamReversingKeyTransformingIterator.class);
 
     TreeMap<Key,Value> expected = new TreeMap<>();
-    for (int row = 1; row <= 3; ++row)
-      for (int cq = 1; cq <= 3; ++cq)
-        for (int cv = 1; cv <= 3; ++cv)
+    for (int row = 1; row <= 3; ++row) {
+      for (int cq = 1; cq <= 3; ++cq) {
+        for (int cv = 1; cv <= 3; ++cv) {
           putExpected(expected, row, expectedCF, cq, cv, PartialKey.ROW);
+        }
+      }
+    }
     checkExpected(expected, "cf2");
   }
 
@@ -290,7 +295,7 @@ public class TransformingIteratorTest {
 
     data.clear();
 
-    Value ev = new Value("".getBytes());
+    Value ev = new Value("");
 
     data.put(new Key("shard001", "foo", "doc02", vis1, 78), ev);
     data.put(new Key("shard001", "dog", "doc02", vis3, 78), ev);
@@ -328,10 +333,13 @@ public class TransformingIteratorTest {
     setUpTransformIterator(ColFamReversingCompactionKeyTransformingIterator.class);
 
     TreeMap<Key,Value> expected = new TreeMap<>();
-    for (int row = 1; row <= 3; ++row)
-      for (int cq = 1; cq <= 3; ++cq)
-        for (int cv = 1; cv <= 3; ++cv)
+    for (int row = 1; row <= 3; ++row) {
+      for (int cq = 1; cq <= 3; ++cq) {
+        for (int cv = 1; cv <= 3; ++cv) {
           putExpected(expected, row, expectedCF, cq, cv, PartialKey.ROW);
+        }
+      }
+    }
     checkExpected(expected, "cf2");
   }
 
@@ -353,7 +361,7 @@ public class TransformingIteratorTest {
             String cv = "badvis";
             long ts = 100 * cfID + 10 * cqID + cvID;
             String val = "val" + ts;
-            expected.put(new Key(row, cf, cq, cv, ts), new Value(val.getBytes()));
+            expected.put(new Key(row, cf, cq, cv, ts), new Value(val));
           }
         }
       }
@@ -365,12 +373,7 @@ public class TransformingIteratorTest {
   @Test
   public void testCompactionAndIllegalVisibility() throws Exception {
     setUpTransformIterator(IllegalVisCompactionKeyTransformingIterator.class);
-    try {
-      checkExpected(new TreeMap<>());
-      fail();
-    } catch (Exception e) {
-
-    }
+    assertThrows(Exception.class, () -> checkExpected(new TreeMap<>()));
   }
 
   @Test
@@ -409,17 +412,11 @@ public class TransformingIteratorTest {
 
     opts.clear();
     opts.put(TransformingIterator.MAX_BUFFER_SIZE_OPT, "A,B");
-    try {
-      ti.validateOptions(opts);
-      fail();
-    } catch (IllegalArgumentException e) {}
+    assertThrows(IllegalArgumentException.class, () -> ti.validateOptions(opts));
 
     opts.clear();
     opts.put(TransformingIterator.AUTH_OPT, Authorizations.HEADER + "~~~~");
-    try {
-      ti.validateOptions(opts);
-      fail();
-    } catch (IllegalArgumentException e) {}
+    assertThrows(IllegalArgumentException.class, () -> ti.validateOptions(opts));
 
   }
 
@@ -447,7 +444,7 @@ public class TransformingIteratorTest {
   private void checkExpected(Range range, Set<ByteSequence> families,
       TreeMap<Key,Value> expectedEntries) throws IOException {
 
-    titer.seek(range, families, families.size() != 0);
+    titer.seek(range, families, !families.isEmpty());
 
     while (titer.hasTop()) {
       Entry<Key,Value> expected = expectedEntries.pollFirstEntry();
@@ -455,13 +452,13 @@ public class TransformingIteratorTest {
       Value actualValue = titer.getTopValue();
       titer.next();
 
-      assertNotNull("Ran out of expected entries on: " + actualKey, expected);
-      assertEquals("Key mismatch", expected.getKey(), actualKey);
-      assertEquals("Value mismatch", expected.getValue(), actualValue);
+      assertNotNull(expected, "Ran out of expected entries on: " + actualKey);
+      assertEquals(expected.getKey(), actualKey, "Key mismatch");
+      assertEquals(expected.getValue(), actualValue, "Value mismatch");
     }
 
-    assertTrue("Scanner did not return all expected entries: " + expectedEntries,
-        expectedEntries.isEmpty());
+    assertTrue(expectedEntries.isEmpty(),
+        "Scanner did not return all expected entries: " + expectedEntries);
   }
 
   private static void putExpected(SortedMap<Key,Value> expected, int rowID, int cfID, int cqID,
@@ -489,7 +486,7 @@ public class TransformingIteratorTest {
       }
     }
 
-    expected.put(new Key(row, cf, cq, cv, ts), new Value(val.getBytes()));
+    expected.put(new Key(row, cf, cq, cv, ts), new Value(val));
   }
 
   private static Text transform(Text val) {
@@ -514,7 +511,7 @@ public class TransformingIteratorTest {
           String val = "val" + ts;
 
           Key k = new Key(row, cf, cq, cv, ts);
-          Value v = new Value(val.getBytes());
+          Value v = new Value(val);
           data.put(k, v);
         }
       }
@@ -604,8 +601,9 @@ public class TransformingIteratorTest {
     protected Collection<ByteSequence>
         untransformColumnFamilies(Collection<ByteSequence> columnFamilies) {
       HashSet<ByteSequence> untransformed = new HashSet<>();
-      for (ByteSequence cf : columnFamilies)
+      for (ByteSequence cf : columnFamilies) {
         untransformed.add(untransformColumnFamily(cf));
+      }
       return untransformed;
     }
 
