@@ -152,12 +152,14 @@ public class FileManager {
 
   public FileManager(ServerContext context, int maxOpen, Cache<String,Long> fileLenCache) {
 
-    if (maxOpen <= 0)
+    if (maxOpen <= 0) {
       throw new IllegalArgumentException("maxOpen <= 0");
+    }
     this.context = context;
     this.fileLenCache = fileLenCache;
 
-    this.filePermits = new Semaphore(maxOpen, false);
+    // Creates a fair semaphore to ensure thread starvation doesn't occur
+    this.filePermits = new Semaphore(maxOpen, true);
     this.maxOpen = maxOpen;
 
     this.openFiles = new HashMap<>();
@@ -264,17 +266,17 @@ public class FileManager {
       long waitTime = System.currentTimeMillis() - start;
 
       if (waitTime >= slowFilePermitMillis) {
-        log.info("Slow file permits request: {} ms, files requested: {}, tablet: {}", waitTime,
-            files.size(), tablet);
+        log.warn("Slow file permits request: {} ms, files requested: {}, "
+            + "max open files: {}, tablet: {}", waitTime, files.size(), maxOpen, tablet);
       }
     }
 
-    // now that the we are past the semaphore, we have the authority
+    // now that we are past the semaphore, we have the authority
     // to open files.size() files
 
     // determine what work needs to be done in sync block
     // but do the work of opening and closing files outside
-    // a synch block
+    // the block
     synchronized (this) {
 
       filesToOpen = takeOpenFiles(files, readersReserved);
@@ -298,8 +300,9 @@ public class FileManager {
     // open any files that need to be opened
     for (String file : filesToOpen) {
       try {
-        if (!file.contains(":"))
+        if (!file.contains(":")) {
           throw new IllegalArgumentException("Expected uri, got : " + file);
+        }
         Path path = new Path(file);
         FileSystem ns = context.getVolumeManager().getFileSystemByPath(path);
         // log.debug("Opening "+file + " path " + path);
@@ -365,14 +368,16 @@ public class FileManager {
 
       for (FileSKVIterator reader : readers) {
         String fileName = reservedReaders.remove(reader);
-        if (!sawIOException)
+        if (!sawIOException) {
           openFiles.computeIfAbsent(fileName, k -> new ArrayList<>())
               .add(new OpenReader(fileName, reader));
+        }
       }
     }
 
-    if (sawIOException)
+    if (sawIOException) {
       closeReaders(readers);
+    }
 
     // decrement the semaphore
     if (!tablet.isMeta()) {
@@ -546,13 +551,15 @@ public class FileManager {
       releaseReaders(tablet, tabletReservedReaders, false);
       tabletReservedReaders.clear();
 
-      for (FileDataSource fds : dataSources)
+      for (FileDataSource fds : dataSources) {
         fds.unsetIterator();
+      }
     }
 
     public synchronized void reattach(SamplerConfigurationImpl samplerConfig) throws IOException {
-      if (!tabletReservedReaders.isEmpty())
+      if (!tabletReservedReaders.isEmpty()) {
         throw new IllegalStateException();
+      }
 
       List<String> files = dataSources.stream().map(x -> x.file).collect(Collectors.toList());
       Map<FileSKVIterator,String> newlyReservedReaders = openFiles(files);
