@@ -22,6 +22,7 @@ import static java.util.Arrays.stream;
 import static java.util.function.Predicate.not;
 
 import java.io.IOException;
+import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashSet;
@@ -37,8 +38,9 @@ import java.util.stream.Stream;
 import org.apache.accumulo.core.Constants;
 import org.apache.accumulo.core.client.TableNotFoundException;
 import org.apache.accumulo.core.data.TableId;
+import org.apache.accumulo.core.gc.GcCandidate;
 import org.apache.accumulo.core.gc.Reference;
-import org.apache.accumulo.core.gc.ReferenceDirectory;
+import org.apache.accumulo.core.metadata.schema.Ample.GcCandidateType;
 import org.apache.accumulo.core.metadata.schema.MetadataSchema.TabletsSection.ServerColumnFamily;
 import org.apache.accumulo.core.trace.TraceUtil;
 import org.apache.accumulo.server.replication.proto.Replication.Status;
@@ -121,13 +123,13 @@ public class GarbageCollectionAlgorithm {
     return relPath;
   }
 
-  private SortedMap<String,String> makeRelative(Collection<String> candidates) {
-    SortedMap<String,String> ret = new TreeMap<>();
+  private SortedMap<String,GcCandidate> makeRelative(Collection<GcCandidate> candidates) {
+    SortedMap<String,GcCandidate> ret = new TreeMap<>();
 
-    for (String candidate : candidates) {
+    for (GcCandidate candidate : candidates) {
       String relPath;
       try {
-        relPath = makeRelative(candidate, 0);
+        relPath = makeRelative(candidate.getPath(), 0);
       } catch (IllegalArgumentException iae) {
         log.warn("Ignoring invalid deletion candidate {}", candidate);
         continue;
@@ -139,8 +141,9 @@ public class GarbageCollectionAlgorithm {
   }
 
   private void removeCandidatesInUse(GarbageCollectionEnvironment gce,
-      SortedMap<String,String> candidateMap) throws InterruptedException {
+      SortedMap<String,GcCandidate> candidateMap) throws InterruptedException {
 
+    List<GcCandidate> candidateEntriesToBeDeleted = new ArrayList<>();
     Set<TableId> tableIdsBefore = gce.getCandidateTableIDs();
     Set<TableId> tableIdsSeen = new HashSet<>();
     Iterator<Reference> iter = gce.getReferences().iterator();
@@ -149,15 +152,16 @@ public class GarbageCollectionAlgorithm {
       tableIdsSeen.add(ref.getTableId());
 
       if (ref.isDirectory()) {
-        var dirReference = (ReferenceDirectory) ref;
-        ServerColumnFamily.validateDirCol(dirReference.getTabletDir());
+        ServerColumnFamily.validateDirCol(ref.getMetadataEntry());
 
-        String dir = "/" + dirReference.tableId + "/" + dirReference.getTabletDir();
+        String dir = "/" + ref.getTableId() + "/" + ref.getMetadataEntry();
 
         dir = makeRelative(dir, 2);
 
-        if (candidateMap.remove(dir) != null) {
-          log.debug("Candidate was still in use: {}", dir);
+        GcCandidate gcTemp = candidateMap.remove(dir);
+        if (gcTemp != null) {
+          log.debug("Directory Candidate was still in use by dir ref: {}", dir);
+          // Do not add dir candidates to candidateEntriesToBeDeleted as they are only created once.
         }
       } else {
         String reference = ref.getMetadataEntry();
@@ -173,23 +177,34 @@ public class GarbageCollectionAlgorithm {
 
         // WARNING: This line is EXTREMELY IMPORTANT.
         // You MUST REMOVE candidates that are still in use
-        if (candidateMap.remove(relativePath) != null) {
-          log.debug("Candidate was still in use: {}", relativePath);
+        GcCandidate gcTemp = candidateMap.remove(relativePath);
+        if (gcTemp != null) {
+          log.debug("File Candidate was still in use: {}", relativePath);
+          // Prevent deletion of candidates that are still in use by scans, because they won't be
+          // recreated once the scan is finished.
+          if (!ref.isScan()) {
+            candidateEntriesToBeDeleted.add(gcTemp);
+          }
         }
 
         String dir = relativePath.substring(0, relativePath.lastIndexOf('/'));
-        if (candidateMap.remove(dir) != null) {
-          log.debug("Candidate was still in use: {}", relativePath);
+        GcCandidate gcT = candidateMap.remove(dir);
+        if (gcT != null) {
+          log.debug("Directory Candidate was still in use by file ref: {}", relativePath);
+          // Do not add dir candidates to candidateEntriesToBeDeleted as they are only created once.
         }
       }
     }
     Set<TableId> tableIdsAfter = gce.getCandidateTableIDs();
     ensureAllTablesChecked(Collections.unmodifiableSet(tableIdsBefore),
         Collections.unmodifiableSet(tableIdsSeen), Collections.unmodifiableSet(tableIdsAfter));
+    if (gce.canRemoveInUseCandidates()) {
+      gce.deleteGcCandidates(candidateEntriesToBeDeleted, GcCandidateType.INUSE);
+    }
   }
 
   private long removeBlipCandidates(GarbageCollectionEnvironment gce,
-      SortedMap<String,String> candidateMap) throws TableNotFoundException {
+      SortedMap<String,GcCandidate> candidateMap) throws TableNotFoundException {
     long blipCount = 0;
     boolean checkForBulkProcessingFiles = candidateMap.keySet().stream().anyMatch(
         relativePath -> relativePath.toLowerCase(Locale.ENGLISH).contains(Constants.BULK_PREFIX));
@@ -271,20 +286,20 @@ public class GarbageCollectionAlgorithm {
   }
 
   protected void confirmDeletesFromReplication(GarbageCollectionEnvironment gce,
-      SortedMap<String,String> candidateMap) {
+      SortedMap<String,GcCandidate> candidateMap) {
     var replicationNeededIterator = gce.getReplicationNeededIterator();
     var candidateMapIterator = candidateMap.entrySet().iterator();
 
     PeekingIterator<Entry<String,Status>> pendingReplication =
         Iterators.peekingIterator(replicationNeededIterator);
-    PeekingIterator<Entry<String,String>> candidates =
+    PeekingIterator<Entry<String,GcCandidate>> candidates =
         Iterators.peekingIterator(candidateMapIterator);
     while (pendingReplication.hasNext() && candidates.hasNext()) {
       Entry<String,Status> pendingReplica = pendingReplication.peek();
-      Entry<String,String> candidate = candidates.peek();
+      Entry<String,GcCandidate> candidate = candidates.peek();
 
       String filePendingReplication = pendingReplica.getKey();
-      String fullPathCandidate = candidate.getValue();
+      String fullPathCandidate = candidate.getValue().getPath();
 
       int comparison = filePendingReplication.compareTo(fullPathCandidate);
       if (comparison < 0) {
@@ -309,7 +324,7 @@ public class GarbageCollectionAlgorithm {
   }
 
   private void cleanUpDeletedTableDirs(GarbageCollectionEnvironment gce,
-      SortedMap<String,String> candidateMap) throws InterruptedException, IOException {
+      SortedMap<String,GcCandidate> candidateMap) throws InterruptedException, IOException {
     HashSet<TableId> tableIdsWithDeletes = new HashSet<>();
 
     // find the table ids that had dirs deleted
@@ -334,7 +349,8 @@ public class GarbageCollectionAlgorithm {
   }
 
   private long confirmDeletesTrace(GarbageCollectionEnvironment gce,
-      SortedMap<String,String> candidateMap) throws InterruptedException, TableNotFoundException {
+      SortedMap<String,GcCandidate> candidateMap)
+      throws InterruptedException, TableNotFoundException {
     long blips = 0;
     Span confirmDeletesSpan = TraceUtil.startSpan(this.getClass(), "confirmDeletes");
     try (Scope scope = confirmDeletesSpan.makeCurrent()) {
@@ -351,7 +367,7 @@ public class GarbageCollectionAlgorithm {
   }
 
   private void deleteConfirmedCandidates(GarbageCollectionEnvironment gce,
-      SortedMap<String,String> candidateMap)
+      SortedMap<String,GcCandidate> candidateMap)
       throws InterruptedException, IOException, TableNotFoundException {
     Span deleteSpan = TraceUtil.startSpan(this.getClass(), "deleteFiles");
     try (Scope deleteScope = deleteSpan.makeCurrent()) {
@@ -369,11 +385,11 @@ public class GarbageCollectionAlgorithm {
   public long collect(GarbageCollectionEnvironment gce)
       throws InterruptedException, TableNotFoundException, IOException {
 
-    Iterator<String> candidatesIter = gce.getCandidates();
+    Iterator<GcCandidate> candidatesIter = gce.getCandidates();
     long totalBlips = 0;
 
     while (candidatesIter.hasNext()) {
-      List<String> batchOfCandidates;
+      List<GcCandidate> batchOfCandidates;
       Span candidatesSpan = TraceUtil.startSpan(this.getClass(), "getCandidates");
       try (Scope candidatesScope = candidatesSpan.makeCurrent()) {
         batchOfCandidates = gce.readCandidatesThatFitInMemory(candidatesIter);
@@ -391,13 +407,13 @@ public class GarbageCollectionAlgorithm {
   /**
    * Given a sub-list of possible deletion candidates, process and remove valid deletion candidates.
    */
-  private long deleteBatch(GarbageCollectionEnvironment gce, List<String> currentBatch)
+  private long deleteBatch(GarbageCollectionEnvironment gce, List<GcCandidate> currentBatch)
       throws InterruptedException, TableNotFoundException, IOException {
 
     long origSize = currentBatch.size();
     gce.incrementCandidatesStat(origSize);
 
-    SortedMap<String,String> candidateMap = makeRelative(currentBatch);
+    SortedMap<String,GcCandidate> candidateMap = makeRelative(currentBatch);
 
     long blips = confirmDeletesTrace(gce, candidateMap);
     gce.incrementInUseStat(origSize - candidateMap.size());
@@ -406,5 +422,4 @@ public class GarbageCollectionAlgorithm {
 
     return blips;
   }
-
 }
