@@ -528,57 +528,71 @@ public class TabletClientHandler implements TabletClientService.Iface {
 
   @Override
   public UpdateErrors closeUpdate(TInfo tinfo, long updateID) throws NoSuchScanIDException {
-    final UpdateSession us = (UpdateSession) server.sessionManager.removeSession(updateID);
+    // Reserve the session and wait for any write that may currently have it reserved. Once reserved
+    // no write stragglers can start against this session id.
+    final UpdateSession us = (UpdateSession) server.sessionManager.reserveSession(updateID, true);
     if (us == null) {
       throw new NoSuchScanIDException();
     }
 
-    // clients may or may not see data from an update session while
-    // it is in progress, however when the update session is closed
-    // want to ensure that reads wait for the write to finish
-    long opid = writeTracker.startWrite(us.queuedMutations.keySet());
-
     try {
-      flush(us);
-    } catch (HoldTimeoutException e) {
-      // Assumption is that the client has timed out and is gone. If that's not the case throw an
-      // exception that will cause it to retry.
-      log.debug("HoldTimeoutException during closeUpdate, reporting no such session");
-      throw new NoSuchScanIDException();
-    } finally {
-      writeTracker.finishWrite(opid);
-    }
+      // clients may or may not see data from an update session while
+      // it is in progress, however when the update session is closed
+      // want to ensure that reads wait for the write to finish
+      long opid = writeTracker.startWrite(us.queuedMutations.keySet());
 
-    if (log.isTraceEnabled()) {
-      log.trace(
-          String.format("UpSess %s %,d in %.3fs, at=[%s] ft=%.3fs(pt=%.3fs lt=%.3fs ct=%.3fs)",
-              TServerUtils.clientAddress.get(), us.totalUpdates,
-              (System.currentTimeMillis() - us.startTime) / 1000.0, us.authTimes,
-              us.flushTime / 1000.0, us.prepareTimes.sum() / 1000.0, us.walogTimes.sum() / 1000.0,
-              us.commitTimes.sum() / 1000.0));
+      try {
+        flush(us);
+      } catch (HoldTimeoutException e) {
+        // Assumption is that the client has timed out and is gone. If that's not the case throw an
+        // exception that will cause it to retry.
+        log.debug("HoldTimeoutException during closeUpdate, reporting no such session");
+        throw new NoSuchScanIDException();
+      } finally {
+        writeTracker.finishWrite(opid);
+      }
+
+      if (log.isTraceEnabled()) {
+        log.trace(
+            String.format("UpSess %s %,d in %.3fs, at=[%s] ft=%.3fs(pt=%.3fs lt=%.3fs ct=%.3fs)",
+                TServerUtils.clientAddress.get(), us.totalUpdates,
+                (System.currentTimeMillis() - us.startTime) / 1000.0, us.authTimes,
+                us.flushTime / 1000.0, us.prepareTimes.sum() / 1000.0, us.walogTimes.sum() / 1000.0,
+                us.commitTimes.sum() / 1000.0));
+      }
+      if (!us.failures.isEmpty()) {
+        Entry<KeyExtent,Long> first = us.failures.entrySet().iterator().next();
+        log.debug(String.format("Failures: %d, first extent %s successful commits: %d",
+            us.failures.size(), first.getKey().toString(), first.getValue()));
+      }
+      List<ConstraintViolationSummary> violations = us.violations.asList();
+      if (!violations.isEmpty()) {
+        ConstraintViolationSummary first = us.violations.asList().iterator().next();
+        log.debug(String.format("Violations: %d, first %s occurs %d", violations.size(),
+            first.violationDescription, first.numberOfViolatingMutations));
+      }
+      if (!us.authFailures.isEmpty()) {
+        KeyExtent first = us.authFailures.keySet().iterator().next();
+        log.debug(String.format("Authentication Failures: %d, first %s", us.authFailures.size(),
+            first.toString()));
+      }
+      return new UpdateErrors(
+          us.failures.entrySet().stream()
+              .collect(Collectors.toMap(e -> e.getKey().toThrift(), Entry::getValue)),
+          violations.stream().map(ConstraintViolationSummary::toThrift)
+              .collect(Collectors.toList()),
+          us.authFailures.entrySet().stream()
+              .collect(Collectors.toMap(e -> e.getKey().toThrift(), Entry::getValue)));
+    } finally {
+      // Atomically unreserve and delete the session. If there any write stragglers, they will fail
+      // after this point.
+      server.sessionManager.removeSession(updateID, true);
     }
-    if (!us.failures.isEmpty()) {
-      Entry<KeyExtent,Long> first = us.failures.entrySet().iterator().next();
-      log.debug(String.format("Failures: %d, first extent %s successful commits: %d",
-          us.failures.size(), first.getKey().toString(), first.getValue()));
-    }
-    List<ConstraintViolationSummary> violations = us.violations.asList();
-    if (!violations.isEmpty()) {
-      ConstraintViolationSummary first = us.violations.asList().iterator().next();
-      log.debug(String.format("Violations: %d, first %s occurs %d", violations.size(),
-          first.violationDescription, first.numberOfViolatingMutations));
-    }
-    if (!us.authFailures.isEmpty()) {
-      KeyExtent first = us.authFailures.keySet().iterator().next();
-      log.debug(String.format("Authentication Failures: %d, first %s", us.authFailures.size(),
-          first.toString()));
-    }
-    return new UpdateErrors(
-        us.failures.entrySet().stream()
-            .collect(Collectors.toMap(e -> e.getKey().toThrift(), Entry::getValue)),
-        violations.stream().map(ConstraintViolationSummary::toThrift).collect(Collectors.toList()),
-        us.authFailures.entrySet().stream()
-            .collect(Collectors.toMap(e -> e.getKey().toThrift(), Entry::getValue)));
+  }
+
+  @Override
+  public boolean cancelUpdate(TInfo tinfo, long updateID) throws TException {
+    return server.sessionManager.removeIfNotReserved(updateID);
   }
 
   @Override
@@ -909,27 +923,29 @@ public class TabletClientHandler implements TabletClientService.Iface {
       Map<TKeyExtent,List<TConditionalMutation>> mutations, List<String> symbols)
       throws NoSuchScanIDException, TException {
 
-    ConditionalSession cs = (ConditionalSession) server.sessionManager.reserveSession(sessID);
-
-    if (cs == null || cs.interruptFlag.get()) {
-      throw new NoSuchScanIDException();
-    }
-
-    if (!cs.tableId.equals(MetadataTable.ID) && !cs.tableId.equals(RootTable.ID)) {
-      try {
-        server.resourceManager.waitUntilCommitsAreEnabled();
-      } catch (HoldTimeoutException hte) {
-        // Assumption is that the client has timed out and is gone. If that's not the case throw
-        // an exception that will cause it to retry.
-        log.debug("HoldTimeoutException during conditionalUpdate, reporting no such session");
-        throw new NoSuchScanIDException();
-      }
-    }
-
-    TableId tid = cs.tableId;
-    long opid = writeTracker.startWrite(TabletType.type(new KeyExtent(tid, null, null)));
+    ConditionalSession cs = null;
+    Long opid = null;
 
     try {
+      cs = (ConditionalSession) server.sessionManager.reserveSession(sessID);
+      if (cs == null || cs.interruptFlag.get()) {
+        throw new NoSuchScanIDException();
+      }
+
+      if (!cs.tableId.equals(MetadataTable.ID) && !cs.tableId.equals(RootTable.ID)) {
+        try {
+          server.resourceManager.waitUntilCommitsAreEnabled();
+        } catch (HoldTimeoutException hte) {
+          // Assumption is that the client has timed out and is gone. If that's not the case throw
+          // an exception that will cause it to retry.
+          log.debug("HoldTimeoutException during conditionalUpdate, reporting no such session");
+          throw new NoSuchScanIDException();
+        }
+      }
+
+      TableId tid = cs.tableId;
+      opid = writeTracker.startWrite(TabletType.type(new KeyExtent(tid, null, null)));
+
       // @formatter:off
       Map<KeyExtent, List<ServerConditionalMutation>> updates = mutations.entrySet().stream().collect(Collectors.toMap(
                       entry -> KeyExtent.fromThrift(entry.getKey()),
@@ -954,22 +970,34 @@ public class TabletClientHandler implements TabletClientService.Iface {
       return results;
     } catch (IOException ioe) {
       throw new TException(ioe);
+    } catch (Exception e) {
+      log.warn("Exception returned for conditionalUpdate {}", e);
+      throw e;
     } finally {
-      writeTracker.finishWrite(opid);
-      server.sessionManager.unreserveSession(sessID);
+      if (opid != null) {
+        writeTracker.finishWrite(opid);
+      }
+      if (cs != null) {
+        server.sessionManager.unreserveSession(sessID);
+      }
     }
   }
 
   @Override
   public void invalidateConditionalUpdate(TInfo tinfo, long sessID) {
-    // this method should wait for any running conditional update to complete
-    // after this method returns a conditional update should not be able to start
+    // For the given session, this method should wait for any running conditional update to
+    // complete. After this method returns a conditional update should not be able to start against
+    // this session and nothing should be running.
 
     ConditionalSession cs = (ConditionalSession) server.sessionManager.getSession(sessID);
     if (cs != null) {
+      // Setting this may cause anything running to fail. Setting this will prevent anything from
+      // starting.
       cs.interruptFlag.set(true);
     }
 
+    // If a thread is currently running and working on the update, then this should block until it
+    // un-reserves the session.
     cs = (ConditionalSession) server.sessionManager.reserveSession(sessID, true);
     if (cs != null) {
       server.sessionManager.removeSession(sessID, true);
@@ -985,7 +1013,7 @@ public class TabletClientHandler implements TabletClientService.Iface {
   public void splitTablet(TInfo tinfo, TCredentials credentials, TKeyExtent tkeyExtent,
       ByteBuffer splitPoint) throws NotServingTabletException, ThriftSecurityException {
 
-    TableId tableId = TableId.of(new String(ByteBufferUtil.toBytes(tkeyExtent.table)));
+    TableId tableId = TableId.of(new String(ByteBufferUtil.toBytes(tkeyExtent.table), UTF_8));
     NamespaceId namespaceId = getNamespaceId(credentials, tableId);
 
     if (!security.canSplitTablet(credentials, tableId, namespaceId)) {
