@@ -38,7 +38,9 @@ import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentSkipListSet;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicLong;
+import java.util.function.BooleanSupplier;
 import java.util.function.Consumer;
+import java.util.function.Predicate;
 import java.util.function.Supplier;
 import java.util.stream.Collectors;
 
@@ -80,6 +82,7 @@ import org.apache.zookeeper.KeeperException.NoNodeException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import com.google.common.base.Suppliers;
 import com.google.common.collect.Collections2;
@@ -104,12 +107,12 @@ public class CompactableImpl implements Compactable {
 
   private final FileManager fileMgr;
 
-  private Set<CompactionJob> runningJobs = new HashSet<>();
+  private final Set<CompactionJob> runningJobs = new HashSet<>();
   private volatile boolean compactionRunning = false;
 
-  private Supplier<Set<CompactionServiceId>> servicesInUse;
+  private final Supplier<Set<CompactionServiceId>> servicesInUse;
 
-  private Set<CompactionServiceId> servicesUsed = new ConcurrentSkipListSet<>();
+  private final Set<CompactionServiceId> servicesUsed = new ConcurrentSkipListSet<>();
 
   enum ChopSelectionStatus {
     SELECTING, SELECTED, NOT_ACTIVE, MARKING
@@ -124,21 +127,21 @@ public class CompactableImpl implements Compactable {
   private Long compactionId;
   private CompactionConfig compactionConfig;
 
-  private CompactionManager manager;
+  private final CompactionManager manager;
 
   AtomicLong lastSeenCompactionCancelId = new AtomicLong(Long.MIN_VALUE);
 
   private volatile boolean closed = false;
 
-  private Map<ExternalCompactionId,ExternalCompactionInfo> externalCompactions =
+  private final Map<ExternalCompactionId,ExternalCompactionInfo> externalCompactions =
       new ConcurrentHashMap<>();
 
-  private Set<ExternalCompactionId> externalCompactionsCommitting = new HashSet<>();
+  private final Set<ExternalCompactionId> externalCompactionsCommitting = new HashSet<>();
 
   // This interface exists for two purposes. First it allows abstraction of new and old
   // implementations for user pluggable file selection code. Second it facilitates placing code
   // outside of this class.
-  public static interface CompactionHelper {
+  public interface CompactionHelper {
     Set<StoredTabletFile> selectFiles(SortedMap<StoredTabletFile,DataFileValue> allFiles);
 
     Set<StoredTabletFile> getFilesToDrop();
@@ -193,17 +196,17 @@ public class CompactableImpl implements Compactable {
     // important to track this in order to know if the last compaction is a full compaction and
     // should not propagate deletes.
     private boolean initiallySelectedAll = false;
-    private Set<StoredTabletFile> selectedFiles = new HashSet<>();
+    private final Set<StoredTabletFile> selectedFiles = new HashSet<>();
 
     protected Set<StoredTabletFile> allCompactingFiles = new HashSet<>();
 
     // track files produced by compactions of this tablet, those are considered chopped
-    private Set<StoredTabletFile> choppedFiles = new HashSet<>();
+    private final Set<StoredTabletFile> choppedFiles = new HashSet<>();
     private ChopSelectionStatus chopStatus = ChopSelectionStatus.NOT_ACTIVE;
-    private Set<StoredTabletFile> allFilesWhenChopStarted = new HashSet<>();
+    private final Set<StoredTabletFile> allFilesWhenChopStarted = new HashSet<>();
 
     private final KeyExtent extent;
-    private Deriver<Duration> selectionExpirationDeriver;
+    private final Deriver<Duration> selectionExpirationDeriver;
 
     public FileManager(KeyExtent extent, Collection<StoredTabletFile> extCompactingFiles,
         Optional<SelectedInfo> extSelInfo, Deriver<Duration> selectionExpirationDeriver) {
@@ -230,6 +233,11 @@ public class CompactableImpl implements Compactable {
       return selectKind;
     }
 
+    @VisibleForTesting
+    Set<StoredTabletFile> getSelectedFiles() {
+      return Set.copyOf(selectedFiles);
+    }
+
     SelectedInfo getReservedInfo() {
       Preconditions.checkState(selectStatus == FileSelectionStatus.RESERVED);
       return new SelectedInfo(initiallySelectedAll, selectedFiles, selectKind);
@@ -239,12 +247,32 @@ public class CompactableImpl implements Compactable {
 
     protected abstract long getNanoTime();
 
-    boolean initiateSelection(CompactionKind kind) {
+    /**
+     * @return the last id of the last successful user compaction
+     */
+    protected abstract long getLastCompactId();
 
-      Preconditions.checkArgument(kind == CompactionKind.SELECTOR || kind == CompactionKind.USER);
+    boolean initiateSelection(CompactionKind kind, Long compactionId) {
+
+      Preconditions.checkArgument(
+          kind == CompactionKind.SELECTOR && compactionId == null
+              || kind == CompactionKind.USER && compactionId != null,
+          "Unexpected kind and/or compaction id: %s %s", kind, compactionId);
 
       if (selectStatus == FileSelectionStatus.NOT_ACTIVE || (kind == CompactionKind.USER
-          && selectKind == CompactionKind.SELECTOR && noneRunning(CompactionKind.SELECTOR))) {
+          && selectKind == CompactionKind.SELECTOR && noneRunning(CompactionKind.SELECTOR)
+          && selectStatus != FileSelectionStatus.SELECTING)) {
+
+        // Check compaction id when a lock is held and no other user compactions have files
+        // selected, at this point the results of any previous user compactions should be seen. If
+        // user compaction is currently running, then will not get this far because of the checks a
+        // few lines up.
+        if (kind == CompactionKind.USER && getLastCompactId() >= compactionId) {
+          // This user compaction has already completed, so no need to initiate selection of files
+          // for user compaction.
+          return false;
+        }
+
         selectStatus = FileSelectionStatus.NEW;
         selectKind = kind;
         selectedFiles.clear();
@@ -306,8 +334,8 @@ public class CompactableImpl implements Compactable {
     }
 
     class ChopSelector {
-      private Set<StoredTabletFile> allFiles;
-      private Set<StoredTabletFile> filesToExamine;
+      private final Set<StoredTabletFile> allFiles;
+      private final Set<StoredTabletFile> filesToExamine;
 
       private ChopSelector(Set<StoredTabletFile> allFiles, Set<StoredTabletFile> filesToExamine) {
         this.allFiles = allFiles;
@@ -719,6 +747,11 @@ public class CompactableImpl implements Compactable {
       protected long getNanoTime() {
         return System.nanoTime();
       }
+
+      @Override
+      protected long getLastCompactId() {
+        return tablet.getLastCompactId();
+      }
     };
   }
 
@@ -1017,16 +1050,25 @@ public class CompactableImpl implements Compactable {
 
     synchronized (this) {
       if (closed) {
+        log.trace("Selection of files was not initiated {} because closed", getExtent());
         return;
       }
 
-      if (fileMgr.initiateSelection(kind)) {
+      if (fileMgr.initiateSelection(kind, compactionId)) {
         this.chelper = localHelper;
         this.compactionId = compactionId;
         this.compactionConfig = compactionConfig;
         log.trace("Selected compaction status changed {} {} {} {}", getExtent(),
             fileMgr.getSelectionStatus(), compactionId, compactionConfig);
       } else {
+        if (kind == CompactionKind.USER) {
+          // Only log for user compaction because this code is only called when one is initiated via
+          // the API call. For other compaction kinds the tserver will keep periodically attempting
+          // to initiate which would result in lots of logs.
+          log.trace(
+              "Selection of files was not initiated {} compactionId:{} selectStatus:{} selectedFiles:{}",
+              getExtent(), this.compactionId, fileMgr.selectStatus, fileMgr.selectedFiles.size());
+        }
         return;
       }
     }
@@ -1064,7 +1106,6 @@ public class CompactableImpl implements Compactable {
 
         manager.compactableChanged(this);
       }
-
     } catch (Exception e) {
       log.error("Failed to select user compaction files {}", getExtent(), e);
     } finally {
@@ -1074,7 +1115,6 @@ public class CompactableImpl implements Compactable {
         }
       }
     }
-
   }
 
   static Collection<String> asFileNames(Set<StoredTabletFile> files) {
@@ -1093,7 +1133,9 @@ public class CompactableImpl implements Compactable {
 
   @SuppressWarnings("removal")
   private boolean isCompactionStratConfigured() {
-    return tablet.getTableConfiguration().isPropertySet(Property.TABLE_COMPACTION_STRATEGY);
+    var strategyClass = tablet.getTableConfiguration().get(Property.TABLE_COMPACTION_STRATEGY);
+    return tablet.getTableConfiguration().isPropertySet(Property.TABLE_COMPACTION_STRATEGY)
+        && strategyClass != null && !strategyClass.isBlank();
   }
 
   @Override
@@ -1140,10 +1182,11 @@ public class CompactableImpl implements Compactable {
     private final Supplier<Boolean> expensiveCheck;
     private final Supplier<Boolean> inexpensiveCheck;
 
-    public CompactionCheck(CompactionServiceId service, CompactionKind kind, Long compactionId) {
-      this.expensiveCheck = Suppliers.memoizeWithExpiration(() -> {
-        return service.equals(getConfiguredService(kind));
-      }, 3, TimeUnit.SECONDS);
+    public CompactionCheck(CompactionServiceId service, CompactionKind kind,
+        BooleanSupplier keepRunning, Long compactionId) {
+      this.expensiveCheck = Suppliers.memoizeWithExpiration(
+          () -> service.equals(getConfiguredService(kind)) && keepRunning.getAsBoolean(), 3,
+          TimeUnit.SECONDS);
       this.inexpensiveCheck = Suppliers.memoizeWithExpiration(() -> {
         if (closed
             || (kind == CompactionKind.USER && lastSeenCompactionCancelId.get() >= compactionId)) {
@@ -1234,10 +1277,10 @@ public class CompactableImpl implements Compactable {
       if (job.getKind() == CompactionKind.USER) {
         cInfo.iters = compactionConfig.getIterators();
         cInfo.checkCompactionId = this.compactionId;
+        cInfo.localCompactionCfg = this.compactionConfig;
       }
 
       cInfo.localHelper = this.chelper;
-      cInfo.localCompactionCfg = this.compactionConfig;
     }
 
     // Check to ensure the tablet actually has these files now that they are reserved. Compaction
@@ -1271,8 +1314,8 @@ public class CompactableImpl implements Compactable {
   }
 
   @Override
-  public void compact(CompactionServiceId service, CompactionJob job, RateLimiter readLimiter,
-      RateLimiter writeLimiter, long queuedTime) {
+  public void compact(CompactionServiceId service, CompactionJob job, BooleanSupplier keepRunning,
+      RateLimiter readLimiter, RateLimiter writeLimiter, long queuedTime) {
 
     Optional<CompactionInfo> ocInfo = reserveFilesForCompaction(service, job);
     if (ocInfo.isEmpty()) {
@@ -1290,7 +1333,7 @@ public class CompactableImpl implements Compactable {
     try {
       TabletLogger.compacting(getExtent(), job, cInfo.localCompactionCfg);
       tablet.incrementStatusMajor();
-      var check = new CompactionCheck(service, kind, cInfo.checkCompactionId);
+      var check = new CompactionCheck(service, kind, keepRunning, cInfo.checkCompactionId);
       TabletFile tmpFileName = tablet.getNextMapFilenameForMajc(cInfo.propagateDeletes);
       var compactEnv = new MajCEnv(kind, check, readLimiter, writeLimiter, cInfo.propagateDeletes);
 
@@ -1399,7 +1442,8 @@ public class CompactableImpl implements Compactable {
           successful = true;
         } catch (Exception e) {
           metaFile = Optional.empty();
-          log.error("Error committing external compaction {}", extCompactionId, e);
+          log.error("Error committing external compaction: id: {}, extent: {}", extCompactionId,
+              getExtent(), e);
           throw new RuntimeException(e);
         } finally {
           completeCompaction(ecInfo.job, ecInfo.meta.getJobFiles(), metaFile, successful);
@@ -1441,7 +1485,7 @@ public class CompactableImpl implements Compactable {
             .mutate();
         completeCompaction(ecInfo.job, ecInfo.meta.getJobFiles(), Optional.empty(), false);
         externalCompactions.remove(ecid);
-        log.debug("Processed external compaction failure {}", ecid);
+        log.debug("Processed external compaction failure: id: {}, extent: {}", ecid, getExtent());
       } else {
         log.debug("Ignoring request to fail external compaction that is unknown {}", ecid);
       }
@@ -1475,7 +1519,7 @@ public class CompactableImpl implements Compactable {
 
       if (dispatcher == null) {
         log.error(
-            "Failed to dispatch compaction {} kind:{} hints:{}, falling back to {} service. Unable to instantiate dispatcher plugin. Check server log.",
+            "Failed to dispatch compaction, no dispatcher. extent:{} kind:{} hints:{}, falling back to {} service. Unable to instantiate dispatcher plugin. Check server log.",
             getExtent(), kind, debugHints, CompactionServicesConfig.DEFAULT_SERVICE);
         return CompactionServicesConfig.DEFAULT_SERVICE;
       }
@@ -1522,7 +1566,8 @@ public class CompactableImpl implements Compactable {
 
       return dispatch.getService();
     } catch (RuntimeException e) {
-      log.error("Failed to dispatch compaction {} kind:{} hints:{}, falling back to {} service.",
+      log.error(
+          "Failed to dispatch compaction due to exception. extent:{} kind:{} hints:{}, falling back to {} service.",
           getExtent(), kind, debugHints, CompactionServicesConfig.DEFAULT_SERVICE, e);
       return CompactionServicesConfig.DEFAULT_SERVICE;
     }
@@ -1546,7 +1591,7 @@ public class CompactableImpl implements Compactable {
    * Interrupts and waits for any running compactions. After this method returns, no compactions
    * should be running and none should be able to start.
    */
-  public synchronized void close() {
+  public void close() {
     synchronized (this) {
       if (closed) {
         return;
@@ -1557,11 +1602,19 @@ public class CompactableImpl implements Compactable {
       // Wait while internal jobs are running or external compactions are committing. When
       // chopStatus is MARKING or selectStatus is SELECTING, there may be metadata table writes so
       // wait on those. Do not wait on external compactions that are running.
-      while (runningJobs.stream()
-          .anyMatch(job -> !((CompactionExecutorIdImpl) job.getExecutor()).isExternalId())
+      Predicate<CompactionJob> jobsToWaitFor =
+          job -> !((CompactionExecutorIdImpl) job.getExecutor()).isExternalId();
+      while (runningJobs.stream().anyMatch(jobsToWaitFor)
           || !externalCompactionsCommitting.isEmpty()
           || fileMgr.chopStatus == ChopSelectionStatus.MARKING
           || fileMgr.selectStatus == FileSelectionStatus.SELECTING) {
+
+        log.debug(
+            "Closing {} is waiting on {} running compactions, {} committing external compactions, chop marking {}, file selection {}",
+            getExtent(), runningJobs.stream().filter(jobsToWaitFor).count(),
+            externalCompactionsCommitting.size(), fileMgr.chopStatus == ChopSelectionStatus.MARKING,
+            fileMgr.selectStatus == FileSelectionStatus.SELECTING);
+
         try {
           wait(50);
         } catch (InterruptedException e) {
@@ -1570,7 +1623,6 @@ public class CompactableImpl implements Compactable {
         }
       }
     }
-
     manager.compactableClosed(getExtent(), servicesUsed, externalCompactions.keySet());
   }
 }
