@@ -64,6 +64,7 @@ public class ScanServerTest {
     private KeyExtent extent;
     private TabletResolver resolver;
     private ScanReservation reservation;
+    private boolean systemUser;
 
     protected TestScanServer(ScanServerOpts opts, String[] args) {
       super(opts, args);
@@ -101,6 +102,21 @@ public class ScanServerTest {
       return reservation;
     }
 
+    @Override
+    ScanReservation reserveFilesInstrumented(Map<KeyExtent,List<TRange>> extents) {
+      return reservation;
+    }
+
+    @Override
+    ScanReservation reserveFilesInstrumented(long scanId) {
+      return reservation;
+    }
+
+    @Override
+    protected boolean isSystemUser(TCredentials creds) {
+      return systemUser;
+    }
+
   }
 
   private ThriftScanClientHandler handler;
@@ -125,8 +141,8 @@ public class ScanServerTest {
     TabletResolver resolver = createMock(TabletResolver.class);
 
     TestScanServer ss = partialMockBuilder(TestScanServer.class).createMock();
-    expect(reservation.getFailures()).andReturn(Map.of()).times(2);
     expect(reservation.newTablet(ss, sextent)).andReturn(tablet);
+    expect(reservation.getFailures()).andReturn(Map.of()).anyTimes();
     reservation.close();
     reservation.close();
     expect(handler.startScan(tinfo, tcreds, sextent, trange, tcols, 10, titer, ssio, auths, false,
@@ -142,6 +158,7 @@ public class ScanServerTest {
     ss.resolver = resolver;
     ss.reservation = reservation;
     ss.clientAddress = HostAndPort.fromParts("127.0.0.1", 1234);
+    ss.systemUser = false;
 
     TKeyExtent textent = createMock(TKeyExtent.class);
     InitialScan is = ss.startScan(tinfo, tcreds, textent, trange, tcols, 10, titer, ssio, auths,
@@ -149,7 +166,7 @@ public class ScanServerTest {
     assertEquals(15, is.getScanID());
     ss.continueScan(tinfo, is.getScanID(), 0L);
     ss.closeScan(tinfo, is.getScanID());
-    verify(handler);
+    verify(reservation, handler);
   }
 
   @Test
@@ -171,25 +188,25 @@ public class ScanServerTest {
     Map<String,String> execHints = new HashMap<>();
     ScanReservation reservation = createMock(ScanReservation.class);
 
-    expect(handler.startScan(tinfo, tcreds, textent, trange, tcols, 10, titer, ssio, auths, false,
-        false, 10, tsc, 30L, classLoaderContext, execHints, 0L))
-        .andReturn(new InitialScan(15, null));
+    expect(extent.isMeta()).andReturn(false).anyTimes();
+    expect(extent.toThrift()).andReturn(textent).anyTimes();
     expect(reservation.getFailures()).andReturn(Map.of(textent, ranges));
     reservation.close();
-    expect(handler.continueScan(tinfo, 15, 0L)).andReturn(new ScanResult());
-    handler.closeScan(tinfo, 15);
 
-    replay(handler, reservation);
+    replay(extent, reservation);
 
     TestScanServer ss = partialMockBuilder(TestScanServer.class).createMock();
     ss.extent = extent;
     ss.delegate = handler;
     ss.reservation = reservation;
+    ss.systemUser = false;
 
     assertThrows(NotServingTabletException.class, () -> {
       ss.startScan(tinfo, tcreds, textent, trange, tcols, 10, titer, ssio, auths, false, false, 10,
           tsc, 30L, classLoaderContext, execHints, 0L);
     });
+
+    verify(extent, reservation);
   }
 
   @Test
@@ -223,6 +240,7 @@ public class ScanServerTest {
     };
 
     TestScanServer ss = partialMockBuilder(TestScanServer.class).createMock();
+    expect(extent.isMeta()).andReturn(false).anyTimes();
     expect(reservation.newTablet(ss, extent)).andReturn(tablet);
     expect(reservation.getTabletMetadataExtents()).andReturn(Set.of(extent));
     expect(reservation.getFailures()).andReturn(Map.of());
@@ -233,13 +251,14 @@ public class ScanServerTest {
     expect(handler.continueMultiScan(tinfo, 15, 0L)).andReturn(new MultiScanResult());
     handler.closeMultiScan(tinfo, 15);
 
-    replay(reservation, handler);
+    replay(extent, reservation, handler);
 
     ss.delegate = handler;
     ss.extent = extent;
     ss.resolver = resolver;
     ss.reservation = reservation;
     ss.clientAddress = HostAndPort.fromParts("127.0.0.1", 1234);
+    ss.systemUser = false;
 
     Map<TKeyExtent,List<TRange>> extents = new HashMap<>();
     extents.put(createMock(TKeyExtent.class), ranges);
@@ -249,7 +268,7 @@ public class ScanServerTest {
     ss.continueMultiScan(tinfo, is.getScanID(), 0L);
     assertEquals(15, is.getScanID());
     ss.closeMultiScan(tinfo, is.getScanID());
-    verify(handler);
+    verify(extent, reservation, handler);
   }
 
   @Test
@@ -284,25 +303,24 @@ public class ScanServerTest {
     };
 
     TestScanServer ss = partialMockBuilder(TestScanServer.class).createMock();
-    expect(reservation.newTablet(ss, extent)).andReturn(tablet);
+    expect(extent.isMeta()).andReturn(false).anyTimes();
+    expect(reservation.newTablet(ss, extent)).andReturn(tablet).anyTimes();
     expect(reservation.getTabletMetadataExtents()).andReturn(Set.of());
     expect(reservation.getFailures()).andReturn(Map.of(textent, ranges)).anyTimes();
     reservation.close();
-    reservation.close();
     InitialMultiScan ims = new InitialMultiScan(15, null);
     ims.setResult(new MultiScanResult());
     expect(handler.startMultiScan(tinfo, tcreds, tcols, titer, batch, ssio, auths, false, tsc, 30L,
         classLoaderContext, execHints, resolver, 0L)).andReturn(ims);
-    expect(handler.continueMultiScan(tinfo, 15, 0L)).andReturn(new MultiScanResult());
-    handler.closeMultiScan(tinfo, 15);
 
-    replay(reservation, handler);
+    replay(extent, reservation, handler);
 
     ss.delegate = handler;
     ss.extent = extent;
     ss.resolver = resolver;
     ss.reservation = reservation;
     ss.clientAddress = HostAndPort.fromParts("127.0.0.1", 1234);
+    ss.systemUser = false;
 
     Map<TKeyExtent,List<TRange>> extents = new HashMap<>();
     extents.put(textent, ranges);
@@ -311,6 +329,8 @@ public class ScanServerTest {
     assertEquals(15, is.getScanID());
     assertEquals(0, is.getResult().getFailuresSize());
 
+    verify(extent, reservation, handler);
+
   }
 
   @Test
@@ -344,6 +364,7 @@ public class ScanServerTest {
     ss.delegate = handler;
     ss.resolver = resolver;
     ss.clientAddress = HostAndPort.fromParts("127.0.0.1", 1234);
+    ss.systemUser = false;
 
     assertThrows(TException.class, () -> {
       ss.startMultiScan(tinfo, tcreds, extents, tcols, titer, ssio, auths, false, tsc, 30L,
@@ -352,4 +373,94 @@ public class ScanServerTest {
     verify(handler);
   }
 
+  @Test
+  public void testScanMetaTablesSystemUser() throws Exception {
+    handler = createMock(ThriftScanClientHandler.class);
+
+    TInfo tinfo = createMock(TInfo.class);
+    TCredentials tcreds = createMock(TCredentials.class);
+    KeyExtent sextent = createMock(KeyExtent.class);
+    ScanReservation reservation = createMock(ScanReservation.class);
+    SnapshotTablet tablet = createMock(SnapshotTablet.class);
+    TRange trange = createMock(TRange.class);
+    List<TColumn> tcols = new ArrayList<>();
+    List<IterInfo> titer = new ArrayList<>();
+    Map<String,Map<String,String>> ssio = new HashMap<>();
+    List<ByteBuffer> auths = new ArrayList<>();
+    TSamplerConfiguration tsc = createMock(TSamplerConfiguration.class);
+    String classLoaderContext = new String();
+    Map<String,String> execHints = new HashMap<>();
+    TabletResolver resolver = createMock(TabletResolver.class);
+
+    TestScanServer ss = partialMockBuilder(TestScanServer.class).createMock();
+    expect(sextent.isMeta()).andReturn(true).anyTimes();
+    expect(reservation.newTablet(ss, sextent)).andReturn(tablet);
+    expect(reservation.getFailures()).andReturn(Map.of()).anyTimes();
+    reservation.close();
+    reservation.close();
+    expect(handler.startScan(tinfo, tcreds, sextent, trange, tcols, 10, titer, ssio, auths, false,
+        false, 10, tsc, 30L, classLoaderContext, execHints, resolver, 0L))
+        .andReturn(new InitialScan(15, null));
+    expect(handler.continueScan(tinfo, 15, 0L)).andReturn(new ScanResult());
+    handler.closeScan(tinfo, 15);
+
+    replay(sextent, reservation, handler);
+
+    ss.delegate = handler;
+    ss.extent = sextent;
+    ss.resolver = resolver;
+    ss.reservation = reservation;
+    ss.clientAddress = HostAndPort.fromParts("127.0.0.1", 1234);
+    ss.systemUser = true;
+
+    TKeyExtent textent = createMock(TKeyExtent.class);
+    InitialScan is = ss.startScan(tinfo, tcreds, textent, trange, tcols, 10, titer, ssio, auths,
+        false, false, 10, tsc, 30L, classLoaderContext, execHints, 0L);
+    assertEquals(15, is.getScanID());
+    ss.continueScan(tinfo, is.getScanID(), 0L);
+    ss.closeScan(tinfo, is.getScanID());
+    verify(sextent, reservation, handler);
+
+  }
+
+  @Test
+  public void testScanMetaTablesNonSystemUser() throws Exception {
+    handler = createMock(ThriftScanClientHandler.class);
+
+    TInfo tinfo = createMock(TInfo.class);
+    TCredentials tcreds = createMock(TCredentials.class);
+    KeyExtent sextent = createMock(KeyExtent.class);
+    ScanReservation reservation = createMock(ScanReservation.class);
+    TRange trange = createMock(TRange.class);
+    List<TColumn> tcols = new ArrayList<>();
+    List<IterInfo> titer = new ArrayList<>();
+    Map<String,Map<String,String>> ssio = new HashMap<>();
+    List<ByteBuffer> auths = new ArrayList<>();
+    TSamplerConfiguration tsc = createMock(TSamplerConfiguration.class);
+    String classLoaderContext = new String();
+    Map<String,String> execHints = new HashMap<>();
+    TabletResolver resolver = createMock(TabletResolver.class);
+
+    TestScanServer ss = partialMockBuilder(TestScanServer.class).createMock();
+    expect(sextent.isMeta()).andReturn(true).anyTimes();
+    expect(reservation.getFailures()).andReturn(Map.of()).anyTimes();
+
+    replay(sextent, reservation, handler);
+
+    ss.delegate = handler;
+    ss.extent = sextent;
+    ss.resolver = resolver;
+    ss.reservation = reservation;
+    ss.clientAddress = HostAndPort.fromParts("127.0.0.1", 1234);
+    ss.systemUser = false;
+
+    TKeyExtent textent = createMock(TKeyExtent.class);
+    assertThrows(TException.class, () -> {
+      ss.startScan(tinfo, tcreds, textent, trange, tcols, 10, titer, ssio, auths, false, false, 10,
+          tsc, 30L, classLoaderContext, execHints, 0L);
+    });
+    verify(sextent, reservation, handler);
+
+  }
+
 }
