@@ -25,6 +25,7 @@ import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
+import java.util.Objects;
 import java.util.Set;
 import java.util.TreeMap;
 
@@ -52,8 +53,8 @@ import org.jline.reader.LineReader;
 import com.google.common.collect.ImmutableSortedMap;
 
 public class ConfigCommand extends Command {
-  private Option tableOpt, deleteOpt, setOpt, filterOpt, filterWithValuesOpt, disablePaginationOpt,
-      outputFileOpt, namespaceOpt;
+  private Option tableOpt, deleteOpt, setOpt, forceOpt, filterOpt, filterWithValuesOpt,
+      disablePaginationOpt, outputFileOpt, namespaceOpt;
 
   private int COL1 = 10, COL2 = 7;
   private LineReader reader;
@@ -78,6 +79,8 @@ public class ConfigCommand extends Command {
       NamespaceNotFoundException {
     reader = shellState.getReader();
 
+    boolean force = cl.hasOption(forceOpt);
+
     final String tableName = cl.getOptionValue(tableOpt.getOpt());
     if (tableName != null && !shellState.getAccumuloClient().tableOperations().exists(tableName)) {
       throw new TableNotFoundException(null, tableName, null);
@@ -117,7 +120,8 @@ public class ConfigCommand extends Command {
       }
     } else if (cl.hasOption(setOpt.getOpt())) {
       // set property on table
-      String property = cl.getOptionValue(setOpt.getOpt()), value = null;
+      String property = cl.getOptionValue(setOpt.getOpt());
+      String value;
       if (!property.contains("=")) {
         throw new BadArgumentException("Missing '=' operator in set operation.", fullCommand,
             fullCommand.indexOf(property));
@@ -126,6 +130,17 @@ public class ConfigCommand extends Command {
       property = pair[0];
       value = pair[1];
 
+      // check for deprecation
+      var theProp = Property.getPropertyByKey(property);
+      if (theProp != null && theProp.isDeprecated()) {
+        if (!force
+            && !shellState.confirm("Trying to set deprecated property `" + property + "` continue")
+                .orElse(false)) {
+          throw new BadArgumentException(
+              "Tried to set deprecated property and force not specified.", fullCommand,
+              fullCommand.indexOf(property));
+        }
+      }
       if (tableName != null) {
         if (!Property.isValidTablePropertyKey(property)) {
           throw new BadArgumentException("Invalid per-table property.", fullCommand,
@@ -199,8 +214,8 @@ public class ConfigCommand extends Command {
         String n = Namespaces.getNamespaceName(shellState.getContext(),
             shellState.getContext().getNamespaceId(shellState.getContext().getTableId(tableName)));
         try {
-          shellState.getAccumuloClient().namespaceOperations().getConfiguration(n)
-              .forEach(namespaceConfig::put);
+          namespaceConfig
+              .putAll(shellState.getAccumuloClient().namespaceOperations().getConfiguration(n));
         } catch (AccumuloSecurityException e) {
           if (e.getSecurityErrorCode() == PERMISSION_DENIED) {
             Shell.log.warn(
@@ -303,7 +318,7 @@ public class ConfigCommand extends Command {
                 siteVal == null ? "" : siteVal);
             printed = true;
           }
-          if (!siteConfig.containsKey(key) || !siteVal.equals(sysVal)) {
+          if (!siteConfig.containsKey(key) || !Objects.equals(siteVal, sysVal)) {
             printConfLine(output, "system", printed ? "   @override" : key, sysVal);
             printed = true;
           }
@@ -316,7 +331,7 @@ public class ConfigCommand extends Command {
             printConfLine(output, "default", key, dfault);
             printed = true;
           }
-          if (!systemConfig.containsKey(key) || !sysVal.equals(nspVal)) {
+          if (!systemConfig.containsKey(key) || !Objects.equals(sysVal, nspVal)) {
             printConfLine(output, "namespace", printed ? "   @override" : key, nspVal);
             printed = true;
           }
@@ -398,6 +413,8 @@ public class ConfigCommand extends Command {
         "table to display/set/delete properties for");
     deleteOpt = new Option("d", "delete", true, "delete a per-table property");
     setOpt = new Option("s", "set", true, "set a per-table property");
+    forceOpt = new Option("force", "force", false,
+        "used with set to set a deprecated property without asking");
     filterOpt = new Option("f", "filter", true,
         "show only properties that contain this string in their name.");
     filterWithValuesOpt = new Option("fv", "filter-with-values", true,
@@ -428,6 +445,7 @@ public class ConfigCommand extends Command {
     o.addOptionGroup(og);
     o.addOption(disablePaginationOpt);
     o.addOption(outputFileOpt);
+    o.addOption(forceOpt);
 
     return o;
   }
@@ -436,4 +454,5 @@ public class ConfigCommand extends Command {
   public int numArgs() {
     return 0;
   }
+
 }
