@@ -1,24 +1,25 @@
 /*
- * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
  *
- *     http://www.apache.org/licenses/LICENSE-2.0
+ *   https://www.apache.org/licenses/LICENSE-2.0
  *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 package org.apache.accumulo.shell.commands;
 
-import java.util.ArrayList;
-import java.util.Iterator;
-import java.util.List;
+import java.util.regex.Pattern;
+import java.util.stream.Stream;
 
 import org.apache.accumulo.core.client.admin.InstanceOperations;
 import org.apache.accumulo.shell.Shell;
@@ -27,8 +28,6 @@ import org.apache.commons.cli.CommandLine;
 import org.apache.commons.cli.Option;
 import org.apache.commons.cli.Options;
 
-import com.google.common.collect.Iterators;
-
 public class ListCompactionsCommand extends Command {
 
   private Option tserverOption, disablePaginationOpt, filterOption;
@@ -36,7 +35,7 @@ public class ListCompactionsCommand extends Command {
   @Override
   public String description() {
     return "lists what compactions are currently running in accumulo. See the"
-        + " accumulo.core.client.admin.ActiveCompaciton javadoc for more information"
+        + " accumulo.core.client.admin.ActiveCompaction javadoc for more information"
         + " about columns.";
   }
 
@@ -44,32 +43,33 @@ public class ListCompactionsCommand extends Command {
   public int execute(final String fullCommand, final CommandLine cl, final Shell shellState)
       throws Exception {
 
-    List<String> tservers;
     String filterText = null;
 
     final InstanceOperations instanceOps = shellState.getAccumuloClient().instanceOperations();
 
     final boolean paginate = !cl.hasOption(disablePaginationOpt.getOpt());
 
+    Stream<String> activeCompactionStream;
+
     if (cl.hasOption(tserverOption.getOpt())) {
-      tservers = new ArrayList<>();
-      tservers.add(cl.getOptionValue(tserverOption.getOpt()));
+      activeCompactionStream = ActiveCompactionHelper
+          .activeCompactionsForServer(cl.getOptionValue(tserverOption.getOpt()), instanceOps);
     } else {
-      tservers = instanceOps.getTabletServers();
+      activeCompactionStream = ActiveCompactionHelper.activeCompactions(instanceOps);
     }
 
     if (cl.hasOption(filterOption.getOpt())) {
       filterText = ".*" + cl.getOptionValue(filterOption.getOpt()) + ".*";
     }
 
-    Iterator<String> activeCompactionIterator = new ActiveCompactionIterator(tservers, instanceOps);
     if (filterText != null) {
-      String finalFilterText = filterText;
-      activeCompactionIterator =
-          Iterators.filter(activeCompactionIterator, t -> t.matches(finalFilterText));
+      activeCompactionStream =
+          activeCompactionStream.filter(Pattern.compile(filterText).asMatchPredicate());
     }
 
-    shellState.printLines(activeCompactionIterator, paginate);
+    activeCompactionStream = ActiveCompactionHelper.appendHeader(activeCompactionStream);
+
+    shellState.printLines(activeCompactionStream.iterator(), paginate);
 
     return 0;
   }
