@@ -21,16 +21,22 @@ package org.apache.accumulo.test.functional;
 import static org.apache.accumulo.core.util.UtilWaitThread.sleepUninterruptibly;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertNotEquals;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.junit.jupiter.api.Assertions.fail;
 
 import java.io.UncheckedIOException;
 import java.time.Duration;
+import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Iterator;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Map.Entry;
 import java.util.concurrent.TimeUnit;
+import java.util.stream.Collectors;
+import java.util.stream.Stream;
 
 import org.apache.accumulo.core.Constants;
 import org.apache.accumulo.core.client.Accumulo;
@@ -45,9 +51,12 @@ import org.apache.accumulo.core.data.Value;
 import org.apache.accumulo.core.fate.zookeeper.ServiceLock;
 import org.apache.accumulo.core.fate.zookeeper.ZooReaderWriter;
 import org.apache.accumulo.core.fate.zookeeper.ZooUtil;
+import org.apache.accumulo.core.gc.GcCandidate;
 import org.apache.accumulo.core.gc.ReferenceFile;
 import org.apache.accumulo.core.metadata.MetadataTable;
+import org.apache.accumulo.core.metadata.StoredTabletFile;
 import org.apache.accumulo.core.metadata.schema.Ample;
+import org.apache.accumulo.core.metadata.schema.Ample.DataLevel;
 import org.apache.accumulo.core.metadata.schema.MetadataSchema.DeletesSection;
 import org.apache.accumulo.core.metadata.schema.MetadataSchema.DeletesSection.SkewedKeyValue;
 import org.apache.accumulo.core.security.Authorizations;
@@ -71,11 +80,14 @@ import org.apache.hadoop.io.Text;
 import org.apache.zookeeper.KeeperException;
 import org.apache.zookeeper.KeeperException.NoNodeException;
 import org.junit.jupiter.api.Test;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import com.google.common.collect.Iterators;
 
 public class GarbageCollectorIT extends ConfigurableMacBase {
   private static final String OUR_SECRET = "itsreallysecret";
+  public static final Logger log = LoggerFactory.getLogger(GarbageCollectorIT.class);
 
   @Override
   protected Duration defaultTimeout() {
@@ -264,6 +276,132 @@ public class GarbageCollectorIT extends ConfigurableMacBase {
     }
   }
 
+  @Test
+  public void testUserUniqueMutationDelete() throws Exception {
+    killMacGc();
+    try (AccumuloClient c = Accumulo.newClient().from(getClientProperties()).build()) {
+      String table = getUniqueNames(1)[0];
+      c.tableOperations().create(table);
+      log.info("User GcCandidate Deletion test of table: {}", table);
+      log.info("GcCandidates will be added/removed from table: {}", DataLevel.USER.metaTable());
+      createAndDeleteUniqueMutation(TableId.of(table), Ample.GcCandidateType.INUSE);
+      createAndDeleteUniqueMutation(TableId.of(table), Ample.GcCandidateType.VALID);
+    }
+  }
+
+  @Test
+  public void testMetadataUniqueMutationDelete() throws Exception {
+    killMacGc();
+    TableId tableId = DataLevel.USER.metaTableId();
+    log.info("Metadata GcCandidate Deletion test");
+    log.info("GcCandidates will be added/removed from table: {}", DataLevel.METADATA.metaTable());
+    createAndDeleteUniqueMutation(tableId, Ample.GcCandidateType.INUSE);
+    createAndDeleteUniqueMutation(tableId, Ample.GcCandidateType.VALID);
+  }
+
+  /**
+   * Root INUSE deletions are not supported in 2.1.x. This test can be migrated to use
+   * createAndDeleteUniqueMutation in 3.x
+   *
+   * @throws Exception may occur when killing the GC process.
+   */
+  @Test
+  public void testRootUniqueMutationDelete() throws Exception {
+    killMacGc();
+    TableId tableId = DataLevel.METADATA.metaTableId();
+    log.info("Root GcCandidate Deletion test");
+    log.info("GcCandidates will be added but not removed from Zookeeper");
+
+    Ample ample = cluster.getServerContext().getAmple();
+    DataLevel datalevel = DataLevel.ROOT;
+
+    // Ensure that no other candidates exist before starting test.
+    Iterator<GcCandidate> cIter = ample.getGcCandidates(datalevel);
+
+    ArrayList<GcCandidate> tempCandidates = new ArrayList<>();
+    while (cIter.hasNext()) {
+      GcCandidate cTemp = cIter.next();
+      tempCandidates.add(cTemp);
+    }
+    if (tempCandidates.size() != 0) {
+      ample.deleteGcCandidates(datalevel, tempCandidates, Ample.GcCandidateType.VALID);
+      tempCandidates.clear();
+
+      cIter = ample.getGcCandidates(datalevel);
+      while (cIter.hasNext()) {
+        GcCandidate cTemp = cIter.next();
+        log.debug("PreExisting Candidate Found: {}", cTemp);
+        tempCandidates.add(cTemp);
+      }
+      assertEquals(0, tempCandidates.size());
+    }
+
+    // Create multiple candidate entries
+    List<GcCandidate> candidates =
+        List.of(new GcCandidate("hdfs://foo.com:6000/user/foo/tables/+r/t-0/F00.rf", 0L),
+            new GcCandidate("hdfs://foo.com:6000/user/foo/tables/+r/t-0/F001.rf", 1L));
+
+    List<StoredTabletFile> stfs = new LinkedList<>();
+    candidates.stream().forEach(temp -> stfs.add(new StoredTabletFile(temp.getPath())));
+
+    log.debug("Adding root table GcCandidates");
+    ample.putGcCandidates(tableId, stfs);
+
+    // Retrieve the recently created entries.
+    cIter = ample.getGcCandidates(datalevel);
+
+    int counter = 0;
+    while (cIter.hasNext()) {
+      // Duplicate these entries back into zookeeper
+      ample.putGcCandidates(tableId, List.of(new StoredTabletFile(cIter.next().getPath())));
+      counter++;
+    }
+    // Ensure Zookeeper collapsed the entries and did not support duplicates.
+    assertEquals(2, counter);
+
+    cIter = ample.getGcCandidates(datalevel);
+    while (cIter.hasNext()) {
+      // This should be a noop call. Root inUse candidate deletions are not supported.
+      ample.deleteGcCandidates(datalevel, List.of(cIter.next()), Ample.GcCandidateType.INUSE);
+    }
+
+    // Check that GcCandidates still exist
+    cIter = ample.getGcCandidates(datalevel);
+
+    counter = candidates.size();
+    while (cIter.hasNext()) {
+      GcCandidate gcC = cIter.next();
+      log.debug("Candidate Found: {}", gcC);
+      for (GcCandidate cand : candidates) {
+        if (gcC.getPath().equals(cand.getPath())) {
+          // Candidate uid's will never match as they are randomly generated in 2.1.x
+          assertNotEquals(gcC.getUid(), cand.getUid());
+          counter--;
+        }
+      }
+    }
+    // Ensure that we haven't seen more candidates than we expected.
+    assertEquals(0, counter);
+
+    // Delete the candidates as VALID GcCandidates
+    cIter = ample.getGcCandidates(datalevel);
+    while (cIter.hasNext()) {
+      ample.deleteGcCandidates(datalevel, List.of(cIter.next()), Ample.GcCandidateType.VALID);
+    }
+    // Ensure the GcCandidates have been removed.
+    cIter = ample.getGcCandidates(datalevel);
+
+    counter = 0;
+    while (cIter.hasNext()) {
+      GcCandidate gcC = cIter.next();
+      if (gcC != null) {
+        log.error("Candidate Found: {}", gcC);
+        counter++;
+      }
+    }
+    assertEquals(0, counter);
+  }
+
   @Test
   public void testProperPortAdvertisement() throws Exception {
 
@@ -324,9 +462,76 @@ public class GarbageCollectorIT extends ConfigurableMacBase {
         String longpath = "aaaaaaaaaabbbbbbbbbbccccccccccddddddddddeeeeeeeeee"
             + "ffffffffffgggggggggghhhhhhhhhhiiiiiiiiiijjjjjjjjjj";
         var path = String.format("file:/%020d/%s", i, longpath);
-        Mutation delFlag = ample.createDeleteMutation(new ReferenceFile(TableId.of("1"), path));
+        Mutation delFlag = ample.createDeleteMutation(ReferenceFile.forFile(TableId.of("1"), path));
         bw.addMutation(delFlag);
       }
     }
   }
+
+  private void createAndDeleteUniqueMutation(TableId tableId, Ample.GcCandidateType type) {
+    int totalCandidates = 2;
+    boolean inUseDelete = true;
+
+    if (type == Ample.GcCandidateType.VALID) {
+      totalCandidates = 1;
+      inUseDelete = false;
+    }
+    Ample ample = cluster.getServerContext().getAmple();
+    DataLevel datalevel = Ample.DataLevel.of(tableId);
+
+    // Ensure that no other candidates exist before starting test.
+    List<GcCandidate> candidates = new ArrayList<>();
+    Iterator<GcCandidate> candidate = ample.getGcCandidates(datalevel);
+
+    while (candidate.hasNext()) {
+      GcCandidate cTemp = candidate.next();
+      log.debug("PreExisting Candidate Found: {}", cTemp);
+      candidates.add(cTemp);
+    }
+    assertEquals(0, candidates.size());
+
+    // Create multiple candidate entries
+    List<StoredTabletFile> stfs = Stream
+        .of(new StoredTabletFile("hdfs://foo.com:6000/user/foo/tables/a/t-0/F00.rf"),
+            new StoredTabletFile("hdfs://foo.com:6000/user/foo/tables/b/t-0/F00.rf"))
+        .collect(Collectors.toList());
+
+    log.debug("Adding candidates to table {}", tableId);
+    ample.putGcCandidates(tableId, stfs);
+    // Retrieve new entries.
+    candidate = ample.getGcCandidates(datalevel);
+
+    while (candidate.hasNext()) {
+      GcCandidate cTemp = candidate.next();
+      log.debug("Candidate Found: {}", cTemp);
+      candidates.add(cTemp);
+    }
+    assertEquals(2, candidates.size());
+
+    GcCandidate deleteCandidate = candidates.get(0);
+    assertNotNull(deleteCandidate);
+    ample.putGcCandidates(tableId, List.of(new StoredTabletFile(deleteCandidate.getPath())));
+
+    log.debug("Deleting Candidate {}", deleteCandidate);
+    ample.deleteGcCandidates(datalevel, List.of(deleteCandidate), type);
+
+    candidate = ample.getGcCandidates(datalevel);
+
+    int counter = 0;
+    boolean foundNewCandidate = false;
+    while (candidate.hasNext()) {
+      GcCandidate gcC = candidate.next();
+      log.debug("Candidate Found: {}", gcC);
+      if (gcC.getPath().equals(deleteCandidate.getPath())) {
+        assertNotEquals(gcC.getUid(), deleteCandidate.getUid());
+        foundNewCandidate = true;
+      }
+      counter++;
+    }
+    assertEquals(totalCandidates, counter);
+    assertEquals(inUseDelete, foundNewCandidate);
+
+    // Cleanup from test
+    ample.deleteGcCandidates(datalevel, candidates, Ample.GcCandidateType.VALID);
+  }
 }
