@@ -31,6 +31,7 @@ import java.nio.file.Files;
 import java.text.DateFormat;
 import java.text.SimpleDateFormat;
 import java.time.Duration;
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Date;
 import java.util.List;
@@ -402,19 +403,21 @@ public class ShellIT extends SharedMiniClusterBase {
 
   @Test
   public void scanTimestampTest() throws IOException {
+    String name = getUniqueNames(1)[0];
     Shell.log.debug("Starting scanTimestamp test ------------------------");
-    exec("createtable test", true);
+    exec("createtable " + name, true);
     exec("insert r f q v -ts 0", true);
     exec("scan -st", true, "r f:q [] 0\tv");
     exec("scan -st -f 0", true, " : [] 0\t");
     exec("deletemany -f", true);
-    exec("deletetable test -f", true, "Table: [test] has been deleted");
+    exec("deletetable " + name + " -f", true, "Table: [" + name + "] has been deleted");
   }
 
   @Test
   public void scanFewTest() throws IOException {
     Shell.log.debug("Starting scanFew test ------------------------");
-    exec("createtable test", true);
+    String name = getUniqueNames(1)[0];
+    exec("createtable " + name, true);
     // historically, showing few did not pertain to ColVis or Timestamp
     exec("insert 1 123 123456 -l '12345678' -ts 123456789 1234567890", true);
     exec("setauths -s 12345678", true);
@@ -427,7 +430,7 @@ public class ShellIT extends SharedMiniClusterBase {
     exec("scan -st -f 5 -fm org.apache.accumulo.core.util.format.BinaryFormatter", true,
         expectedFew);
     exec("setauths -c", true);
-    exec("deletetable test -f", true, "Table: [test] has been deleted");
+    exec("deletetable " + name + " -f", true, "Table: [" + name + "] has been deleted");
   }
 
   @Test
@@ -462,6 +465,7 @@ public class ShellIT extends SharedMiniClusterBase {
     String expected = "r f:q [vis]\tv";
     String expectedTimestamp = "r f:q [vis] 0\tv";
     exec("grep", false, "No terms specified");
+    exec("grep vis", true, expected);
     exec("grep non_matching_string", true, "");
     // historically, showing few did not pertain to ColVis or Timestamp
     exec("grep r", true, expected);
@@ -472,6 +476,54 @@ public class ShellIT extends SharedMiniClusterBase {
     exec("deletetable t -f", true, "Table: [t] has been deleted");
   }
 
+  @Test
+  void configPropertyTest() throws IOException {
+    final String table = "testtable";
+    final String filterProperty = "config -t " + table + " -f ";
+    final String setProperty = "config -t " + table + " -s ";
+    List<String> expectedStrings = new ArrayList<>();
+
+    try {
+      exec("createtable " + table);
+
+      exec(filterProperty + "table.iterator.scan.vers.opt.maxVersions", true,
+          "table      | table.iterator.scan.vers.opt.maxVersions .. | 1\n");
+      // set to new value and verify results
+      exec(setProperty + "table.iterator.scan.vers.opt.maxVersions=2", true);
+      exec(filterProperty + "table.iterator.scan.vers.opt.maxVersions", true,
+          "table      | table.iterator.scan.vers.opt.maxVersions .. | 2\n");
+      // set to empty string and verify property still exists and is not deleted
+      exec(setProperty + "table.iterator.scan.vers.opt.maxVersions=", true);
+      exec(filterProperty + "table.iterator.scan.vers.opt.maxVersions", true,
+          "table      | table.iterator.scan.vers.opt.maxVersions .. | \n");
+
+      exec(filterProperty + "table.bloom.enabled", true,
+          "default    | table.bloom.enabled ....................... | false\n");
+      exec(setProperty + "table.bloom.enabled=true", true);
+      expectedStrings.add("default    | table.bloom.enabled ....................... | false\n");
+      expectedStrings.add("table      |    @override .............................. | true\n");
+      execExpectList(filterProperty + "table.bloom.enabled", true, expectedStrings);
+      // can't set prop to empty value since type is Boolean
+      exec(setProperty + "table.bloom.enabled=failsSinceNotABoolean", false);
+
+      exec(filterProperty + "table.file.compress.type", true,
+          "default    | table.file.compress.type .................. | gz\n");
+      exec(setProperty + "table.file.compress.type=zippy", true);
+      expectedStrings.clear();
+      expectedStrings.add("default    | table.file.compress.type .................. | gz\n");
+      expectedStrings.add("table      |    @override .............................. | zippy");
+      execExpectList(filterProperty + "table.file.compress.type", true, expectedStrings);
+      exec(setProperty + "table.file.compress.type=", true);
+      expectedStrings.clear();
+      expectedStrings.add("default    | table.file.compress.type .................. | gz\n");
+      expectedStrings.add("table      |    @override .............................. | \n");
+      execExpectList(filterProperty + "table.file.compress.type", true, expectedStrings);
+
+    } finally {
+      exec("deletetable " + table + " -f", true);
+    }
+  }
+
   @Test
   void configTest() throws IOException {
     Shell.log.debug("Starting config property type test -------------------------");
@@ -494,6 +546,7 @@ public class ShellIT extends SharedMiniClusterBase {
         case PATH:
         case PREFIX:
         case STRING:
+        case JSON:
           Shell.log.debug("Skipping " + propertyType + " Property Types");
           continue;
         case TIMEDURATION:
@@ -712,5 +765,4 @@ public class ShellIT extends SharedMiniClusterBase {
     exec("getsplits -m 0", true,
         "0\n1\n2\n3\n4\n5\n6\n7\n8\n9\na\nb\nc\nd\ne\nf\ng\nh\ni\nj\nk\nl\nm\nn\no\np\nq\nr\ns\nt\n");
   }
-
 }
